/* ============================================================================
* Language: C++ 
* File: merlin.cc
* Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
* Sub-package: Stand Alone Single FICO Core Inversion Client. 
*
* Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/06.
*  
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* You can contact University Corporation for Atmospheric Research at
* 3080 Center Green Drive, Boulder, CO 80301

* (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
* Please read the full copyright statement in the file COPYRIGHT_UCAR.
*
=========================================================================== */



#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <errno.h>

#include <string>
#include <iostream>
#include <fstream>


#include "MERLIN_Plan.h"
#include "FileInversion.h"
#include "InitialGuessFilenetCDF.h"

#include "../config.h"

extern int errno;

using namespace std;

// This variable lets the client control debug mode
bool merlin_client_debug_= false;
// Client Name
const char* program_name_;

inline void showUsage()
{
    cout<<PACKAGE_NAME<<": -h -v -x [XML_INPUT_FILE_NAME] -d [DEBUG LEVEL] -p"<<endl;
    cout<<"where"<<endl;
    cout<<"-d [DEBUG LEVEL]: set program to debug mode. This flag is optional.Debug level can be 1 or .2"<<endl;
    cout<<"-x [XML_INPUT_FILE_NAME]: provides the name of an XML file with the instructions about the inversion."<<endl;
    cout<<"-v: show program version in standard output and exits. This flag is optional."<<endl;
    cout<<"-p: Dump both, the observed and the fitted profiles. This flag is optional."<<endl; 
    cout<<"-h: writes this help and exits."<<endl;
    cout<<"Consult the manual for complete documentation."<<endl;
    exit(1);
}

inline void showVersion()
{
    cout<<PACKAGE_NAME<<": "<<PACKAGE_VERSION<<endl;
    cout<<8*sizeof(void *)<<" bit mode"<<endl;
    cout<<"compiled and linked on "<<__DATE__<<" at "<<__TIME__<<endl;
    exit(0);
}



// load the named file and dump its structure to STDOUT
void execute_inversion_plan(MERLIN_Plan &plan)
{
    if (merlin_client_debug_)
    {
        cout<<"------ This is the model as loaded from the plan, before obtaining initial guess data ---------"<<endl;
        cout<<plan.model<<endl;
        cout<<"---------------------------- Other data for this inversion follows ----------------------------"<<endl;
        // cout<<"CSAC::SVD_threshold: "<<CSAC::SVD_threshold<<endl;
        cout<<"CSAC::worst_tolerated_condition_number: "<<CSAC::worst_tolerated_condition_number<<endl;
        cout<<"CSAC::max_damping_parameter: "<<CSAC::max_damping_parameter<<endl;
        cout<<"CSAC::min_damping_parameter: "<<CSAC::min_damping_parameter<<endl;
        cout<<"CSAC::max_number_failures: "<<CSAC::max_number_failures<<endl;
        cout<<"CSAC::max_number_iterations: "<<CSAC::max_number_iterations<<endl;
        cout<<"Input file is set to: "<<plan.input_file<<endl;
        cout<<"Output file is set to: "<<plan.output_file<<endl;
        cout<<"Initialization file is set to: "<<plan.guess_file<<endl;
        cout<<"Slit position min is set to: "<<plan.y_min<<endl;
        cout<<"Slit position max is set to: "<<plan.y_max<<endl;
    }
    CSACFile myfile(plan.input_file);
    myfile.open();
    myfile.set_inversion_lambda_ranges(plan.parsed_ranges);
    FileInversion fi;
    
    fi.set_inverter(plan.solver); 
    
    if (plan.include_profiles_in_output)
        fi.set_file_dumper();

    fi.Y_max = plan.y_max;
    fi.Y_min = plan.y_min;
    fi.individuals = plan.genetic.individuals;
    fi.generations = plan.genetic.generations;
    fi.set_model(plan.model);
    fi.set_minimum_polarization(plan. polarization_threshold);

    if (plan.guess_file == string("SELF"))
    {
        if (merlin_client_debug_)
            cout<<"Using SELF Initialization"<<endl;
        fi.run(&myfile, 0, plan.output_file);
    }
    else
    {
        InitialGuessFilenetCDF guess(plan.guess_file);
        if (!guess.open())
        {
            cerr<<"InitialGuessFilenetCDF "<<plan.guess_file<<" can not be opened, exiting application..."<<endl;
            exit(1);
        }
        guess.load_contents();
        fi.run(&myfile, &guess, plan.output_file);

    }
    
    return;
}


// ----------------------------------------------------------------------
// MERLIN Client main() 
// ----------------------------------------------------------------------
int main(int argc, char* argv[])
{
	program_name_ = argv[0];

    // Timers
    struct rusage usage1;
    struct rusage usage2;
    struct timeval result;
	
    bool requested_profiles = false;
    bool provided_xml_input_file = false;
    string xml_input_file;
    
    // Set the debug level to none by default.
    int debug_level=0;
    // Arguments?
    int c;
    
    try
    {
        // Get timer for usage so far, notice that we have done nothing
        if (getrusage(RUSAGE_SELF, &usage1) != 0)
        {
            cerr<<program_name_<<": getrusage failed, it returns: "<<strerror(errno)<<endl;
            return 1;
        }
        // Parse command line arguments
        while( ( c = getopt( argc, argv, "hvpd:x:" ) ) != EOF )
        {
            switch( c )
            {
                case 'h':
                    showUsage();
                    break;
                case 'd':
                    debug_level = atoi(optarg);
                    if ((debug_level != 1) && (debug_level != 2))
                        showUsage();
                        if (debug_level == 1)
                            merlin_client_debug_ = true;
                            if (debug_level == 2)
                                CSAC::merlin_debug = true;
                                cout<<program_name_<<": Running in debug mode, level "<<debug_level<<endl;
                    break;
                case 'v':
                    showVersion();
                    break;
                case 'p':
                    requested_profiles = true;
                    break;
                case 'x':
                    xml_input_file = optarg;
                    provided_xml_input_file = true;
                    break;	
                case '?':
                    showUsage();
                    break;
            }
        }
        
        // Did we get an input file with the -x flag?
        if( provided_xml_input_file)
        {
            if (merlin_client_debug_)
                cout<<"Running instructions from "<<xml_input_file<<endl;
            MERLIN_Plan planObject;
            planObject.build_inversion_plan_from_xml_file(xml_input_file.c_str());
            if (requested_profiles)
                planObject.include_profiles_in_output = true;
            execute_inversion_plan(planObject);
            if (getrusage(RUSAGE_SELF, &usage2) != 0)
            {
                cerr<<"getrusage failed, it returns: "<<strerror(errno)<<endl;
                return 1;
            }
            timeval_subtract (&result, &usage2.ru_utime, &usage1.ru_utime);
            cout<<program_name_<<": "<<planObject.output_file<<": total user time: "<<result.tv_sec<<" seconds and "<<result.tv_usec<<" microseconds"<<endl;
        }
        else
            showUsage();
        
        return 0;
    }
    catch(string &error_string)
    {
        cerr<<program_name_<<": "<<error_string<<endl;
        return 1;
    }
    catch(MERLIN_Plan_Exception & ex)
    {
        cerr<<program_name_<<": It seems like the plan builder did not like your XML file, got exception: \n"<<ex.what()<<endl;
        if (!merlin_client_debug_)
            cerr<<"Tip: You can always run this program with the flag -d 1 to see how your input is parsed."<<endl;
        return 1;
    }
    catch(...)
    {
        cerr<<program_name_<<": there has been an unknown exception"<<endl;
        return 1;
    }
}
