/* ============================================================================
 * Language: C++ 
 * File: MERLIN_AtomicLine.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_Atomicline_h_
#define MERLIN_AtomicLine_h_

#include <vector>

#include "MERLIN.h"
#include "MERLIN_LScoupling.h"
#include "MERLIN_Parameter.h"
using CSAC::MERLIN_Parameter;
#include "MERLIN_ZeemanSplitting.h"

namespace CSAC
{
    /// Abstraction of an atomic line.
 class MERLIN_AtomicLine : public MERLIN_Object
    {
    public:
        /// Central_wav in Angstrom \f$(\AA)\f$, that is \f$10^{-10}\f$ metres
        merlin_real central_wav ;
        /// Inverse of wavelength (in s) \f$ Central\_wav * 1.e-10 / CSAC::CC \f$
        merlin_real inv_wav ;
        /// Relative strengh for this line with respect of ETA0
        merlin_real relative_strength ;
        ///Wavelength shift (km/s)
        MERLIN_Parameter    wavelength_shift ;           
        /// Quantum numbers, low
        MERLIN_LScoupling e_lower ;
        /// Quantum numbers, low
        MERLIN_LScoupling e_upper ;
        /// \brief Zeeman Components.
        /// Zeeman components for a field normalized (B=1).
        /// @see Zeeman_Component.
        vector<MERLIN_ZeemanComponent> comp;
    public:
        /// Default constructor
    MERLIN_AtomicLine() :
        comp(0),
        central_wav( 0.0 ),
        inv_wav( 0.0 ),
        relative_strength( 0.0 )
        {}
        /// Copy constructor
        MERLIN_AtomicLine(const MERLIN_AtomicLine &rhs)
        {
            central_wav=rhs.central_wav ;
            inv_wav=rhs.inv_wav ;
            relative_strength=rhs.relative_strength ;
            wavelength_shift=rhs.wavelength_shift ;           
            e_lower=rhs.e_lower ;
            e_upper=rhs.e_upper ;
            comp.resize(rhs.comp.size());
            for (int h=0; h< rhs.comp.size(); h++)
                comp[h]=rhs.comp[h];
        }
        /// Destructor      
        virtual ~MERLIN_AtomicLine() {}
        /// Printer method
        virtual void dump( ostream &strm ) const ;
        /// Pre-load Zeeman components into private member comp.
        void load_components()
        {
            // Zeeman components 
            CSAC::MERLIN_ZeemanSplitting::compute(central_wav, e_lower, e_upper, comp);
        }
        ///
        MERLIN_AtomicLine& operator=(const MERLIN_AtomicLine &rhs)
        {
            if(this == &rhs)
                return *this;
            central_wav=rhs.central_wav ;
            inv_wav=rhs.inv_wav ;
            relative_strength=rhs.relative_strength ;
            wavelength_shift=rhs.wavelength_shift ;           
            e_lower=rhs.e_lower ;
            e_upper=rhs.e_upper ;
            comp.resize(rhs.comp.size());
            for (int h=0; h< rhs.comp.size(); h++)
                comp[h]=rhs.comp[h];
            return *this;
        }
    
    }; // end class MERLIN_AtomicLine
  
} // end namespace CSAC

#endif //  MERLIN_Atomicline_h_

