/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ConvolveProfile.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#include <float.h>
#include <math.h>

#include <complex>
typedef std::complex<double> Complex;
#include <fftw3.h>

#include <cstdlib>
#include <cstring>
#include <vector>
#include <iostream>
using std::cerr;
using std::cout;
using std::endl;
using std::flush;

#include "MERLIN.h"
#include "MERLIN_Model.h"
using CSAC::MERLIN_Model;
#include "MERLIN_Profile.h"
using CSAC::MERLIN_Profile;

namespace CSAC
{
  
    inline void call_fftw_convolution(vector<double> &vector, double *response, int n, int index, fftw_plan forward_plan, fftw_plan backward_plan, double *in, Complex *out) {
		Complex *fd = new Complex [n/2+1];

		memcpy(in, &vector[index], n*sizeof(double));
		fftw_execute(forward_plan);
		memcpy(fd, out, (n/2+1)*sizeof(Complex));

		memcpy(in, response, n*sizeof(double));
		fftw_execute(forward_plan);

		// C++ knows about multiplying complex numbers
		for(int i=0; i < n/2+1; i++)
			out[i] *= fd[i];

		fftw_execute(backward_plan);
		memcpy(&vector[index], in, n*sizeof(double));

		delete [] fd;
	}

    void MERLIN_ConvolveProfile(
                                MERLIN_Model &model, 
                                MERLIN_Profile &profile,
                                bool derivatives
                                )
    {
        int i, n;
        double Wave;
        double x,px;
        double *gauss=0;
        double *dgauss__dvmac=0;
        double Sum, Sum1;
        bool derivative_for_gaussian=false;
        int index_for_gaussian_derivative;
        double wave_step;
		fftw_plan forward_plan, backward_plan;
		double *in;
		Complex *out;

        // lets get sure that the number of regions times the number of 
        // wavelengths per regions matches the size of the vectors
    
        if ( ( model.n_total_wave != profile.stokes.I.size() ) || 
             ( model.n_total_wave != profile.stokes.Q.size() ) || 
             ( model.n_total_wave != profile.stokes.U.size() ) || 
             ( model.n_total_wave != profile.stokes.V.size() ) )
        {
            cerr<<__FILE__<<":"<<__LINE__<<": The input stokes vectors are not of the same length or mismatch data sizes"<<endl;
            exit(1);
        }
    
        if(derivatives)
        {
            // If we are convolving the derivatives as well, 
            // we must get sure there is a derivative for each wave.
      
            // We get sure that the derivatives loaded in profile.stokes_derivatives were computed 
            // thus it must match the size of profile.stokes_derivatives.order.
      
            if ( profile.stokes_derivatives.I_dxdy.size() != profile.stokes_derivatives.order.size() )
            {
                cerr<<__FILE__<<":"<<__LINE__<<flush;
                cerr<<": The number of derivative vectors does not match the number of derivatives indicated by the order"<<endl;
                cerr<<"profile.stokes_derivatives.I_dxdy.size= "<<profile.stokes_derivatives.I_dxdy.size()<<endl;
                cerr<<"profile.stokes_derivatives.order.size= "<<profile.stokes_derivatives.order.size()<<endl;
                exit(1);
            }
            if ( profile.stokes_derivatives.Q_dxdy.size() != profile.stokes_derivatives.order.size() )
            {
                cerr<<__FILE__<<":"<<__LINE__<<flush;
                cerr<<": The number of derivative vectors does not match the number of derivatives indicated by the order"<<endl;
                cerr<<"profile.stokes_derivatives.Q_dxdy.size= "<<profile.stokes_derivatives.Q_dxdy.size()<<endl;
                cerr<<"profile.stokes_derivatives.order.size= "<<profile.stokes_derivatives.order.size()<<endl;
                exit(1);
            }
            if ( profile.stokes_derivatives.U_dxdy.size() != profile.stokes_derivatives.order.size() )
            {
                cerr<<__FILE__<<":"<<__LINE__<<flush;
                cerr<<": The number of derivative vectors does not match the number of derivatives indicated by the order"<<endl;
                cerr<<"profile.stokes_derivatives.U_dxdy.size= "<<profile.stokes_derivatives.U_dxdy.size()<<endl;
                cerr<<"profile.stokes_derivatives.order.size= "<<profile.stokes_derivatives.order.size()<<endl;
                exit(1);
            }
            if ( profile.stokes_derivatives.V_dxdy.size() != profile.stokes_derivatives.order.size() )
            {
                cerr<<__FILE__<<":"<<__LINE__<<flush;
                cerr<<": The number of derivative vectors does not match the number of derivatives indicated by the order"<<endl;
                cerr<<"profile.stokes_derivatives.V_dxdy.size= "<<profile.stokes_derivatives.V_dxdy.size()<<endl;
                cerr<<"profile.stokes_derivatives.order.size= "<<profile.stokes_derivatives.order.size()<<endl;
                exit(1);
            }
      
            // We need to find if the derivative of the Gaussian representing the macroturbulence is required too
            // besides, if we are required the derivative for the gaussian, we must find where in the profile 
            // is the data structure to hold the data.
      
            if (model.vmac.free) 
            {
                derivative_for_gaussian=true;
                for (int dpos = 0; dpos < profile.stokes_derivatives.order.size(); dpos++)
                {
                    if (profile.stokes_derivatives.order[dpos]=="vmac")
                    {
                        index_for_gaussian_derivative = dpos;
                        break;
                    }
                }
                // If the derivative is required, the derivative for the macroturbulence
                // must be initialized to the Stokes profiles.
                profile.stokes_derivatives.I_dxdy[index_for_gaussian_derivative] = profile.stokes.I;
                profile.stokes_derivatives.Q_dxdy[index_for_gaussian_derivative] = profile.stokes.Q;
                profile.stokes_derivatives.U_dxdy[index_for_gaussian_derivative] = profile.stokes.U;
                profile.stokes_derivatives.V_dxdy[index_for_gaussian_derivative] = profile.stokes.V;
            }
        }
    
        // We must not convolve the vectors for the derivatives of the stray light 
        // and the stray light shift. Lets find out their indexes in the order.
    
        // First the stray light derivatives.
        int index_for_stray_light_derivative;
        if (model.stray_light.free)
        {
            for (int dpos=0; dpos < profile.stokes_derivatives.order.size(); dpos++)
            {
                if(profile.stokes_derivatives.order[dpos]=="stray_light")
                {
                    index_for_stray_light_derivative=dpos;
                    break;
                }
            }
        }
    
        // Second the stray light shift derivatives.
        int index_for_stray_light_shift_derivative;
        if (model.stray_light_shift.free)
        {
            for (int dpos=0; dpos<profile.stokes_derivatives.order.size(); dpos++)
            {
                if(profile.stokes_derivatives.order[dpos]=="stray_light_shift")
                {
                    index_for_stray_light_shift_derivative=dpos;
                    break;
                }
            }
        }
    
    
        // If the macroturbulence is zero or smaller than four times the machine double epsilon
        // then the gaussian is 1 thus the convolution will return the same signal 
        if ( (model.vmac.value == 0.0) || (model.vmac.value < DBL_EPSILON*4) )
            return;
    
        // since we are reading part by part of the stokes vectors this index tells us
        // were we are given the region used
        unsigned int index = 0;
        for (int iregion = 0; iregion < model.n_regions; iregion++) /* Loop on spectral regions */
        {
            wave_step = model.regions[iregion].wave_step * 1E-3;
            n = model.regions[iregion].n_wavelengths;
      
            // Allocate a vector of doubles to build a Gaussian to be convolved with the Stokes vectors.
            // set all values to zero
            gauss = new double [n];
            memset(gauss, 0, (n * sizeof(double)));
      
            // Allocate a vector of doubles to build a derivative of the Gaussian to be convolved with the Stokes vectors.
            if (derivative_for_gaussian)
            {
                dgauss__dvmac = new double [n];
                memset(dgauss__dvmac, 0, (n * sizeof(double)));
            }
      
      
            // Construct the gaussian profile to convolve 
            Wave = model.regions[iregion].first_wavelength + model.regions[iregion].n_wavelengths/2. * wave_step;
      
             // Make sure that n is not greater than what this
             // function aligns to
            if (n > 4096)
            {
                char error_line[32];
                sprintf(error_line, "%i", __LINE__);
                string t_error=__FILE__ + string(":") + string(error_line);
                t_error += string(": \nwhile trying to build a gaussian with the given macroturbulence\n");
                t_error += string("the number of points is to large to make enough zero padding\n");
                throw t_error;
            }    
      
            // Normalize the response function
            Sum=0;
			if (derivative_for_gaussian) Sum1=0;

			for (i=0; i < n/2+1; i++) /* Positive x */
			{
				x = i * wave_step / (model.vmac.value * 1E3 / CSAC::CC * Wave);
				px = i * wave_step / ((model.vmac.value + CSAC::perturbation) * 1E3 / CSAC::CC * Wave);
				gauss[i]=exp(-x*x);
				Sum+=gauss[i];
				if(derivative_for_gaussian)
				{
					dgauss__dvmac[i] = exp(-px*px);
					Sum1+=dgauss__dvmac[i];
				}
			}
			for (i=0; i < n/2; i++) /* Negative x */
			{
				gauss[n-i-1] = gauss[i+1];
				Sum+=gauss[n-i-1];
				if(derivative_for_gaussian)
				{
					dgauss__dvmac[n-i-1] =dgauss__dvmac[i+1];
					Sum1+=dgauss__dvmac[n-i-1];
				}
			}

			// compute numerically the derivative for the Gaussian, apply normalization factor for the perturbed Gaussian
			if (derivative_for_gaussian)
            {
                for (i=0; i < n; i++)
                    dgauss__dvmac[i]=(dgauss__dvmac[i] - gauss[i])/CSAC::perturbation;
            }
          
            // normalization
			// normalize to 1/n because FFT is not normalized
            for (i=0; i < n; i++) 
            {
                gauss[i]=gauss[i]/Sum/n;
                if (derivative_for_gaussian)
                    dgauss__dvmac[i]/Sum1/n;
            }

			// setup FFTW plans
			in = new double [n];
			out = new Complex [n];
			forward_plan = fftw_plan_dft_r2c_1d(n, in, reinterpret_cast<fftw_complex*>(out), FFTW_MEASURE);
			backward_plan = fftw_plan_dft_c2r_1d(n, reinterpret_cast<fftw_complex*>(out), in, FFTW_MEASURE);

			call_fftw_convolution(profile.stokes.I, gauss, n, index, forward_plan, backward_plan, in, out);
			call_fftw_convolution(profile.stokes.Q, gauss, n, index, forward_plan, backward_plan, in, out);
			call_fftw_convolution(profile.stokes.U, gauss, n, index, forward_plan, backward_plan, in, out);
			call_fftw_convolution(profile.stokes.V, gauss, n, index, forward_plan, backward_plan, in, out);

            if (derivatives)
            {
                if (derivative_for_gaussian)
                {
                    for(int f = 0; f < profile.stokes_derivatives.I_dxdy.size(); f++)
                    {
                        if(f != index_for_gaussian_derivative)
                        {
                            if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                            {
                                if (model.n_total_wave != profile.stokes_derivatives.I_dxdy[f].size())
                                {
                                    cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                    cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                    cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.I_dxdy[f].size()<<endl;
                                    exit(1);
                                }
								call_fftw_convolution(profile.stokes_derivatives.I_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                            }
                        }
                    }
                    for(int f = 0; f < profile.stokes_derivatives.Q_dxdy.size(); f++)
                    {
                        if(f!=index_for_gaussian_derivative)
                        {
                            if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                            {
                                if (model.n_total_wave != profile.stokes_derivatives.Q_dxdy[f].size())
                                {
                                    cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                    cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                    cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.Q_dxdy[f].size()<<endl;
                                    exit(1);
                                }
								call_fftw_convolution(profile.stokes_derivatives.Q_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                            }
                        }
                    }
                    for(int f = 0; f < profile.stokes_derivatives.U_dxdy.size(); f++)
                    {
                        if(f!=index_for_gaussian_derivative)
                        {
                            if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                            {
                                if (model.n_total_wave != profile.stokes_derivatives.U_dxdy[f].size())
                                {
                                    cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                    cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                    cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.U_dxdy[f].size()<<endl;
                                    exit(1);
                                }
								call_fftw_convolution(profile.stokes_derivatives.U_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                            }
                        }
                    }
                    for(int f=0; f < profile.stokes_derivatives.V_dxdy.size(); f++)
                    {
                        if(f != index_for_gaussian_derivative)
                        {
                            if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                            {
                                if (model.n_total_wave != profile.stokes_derivatives.V_dxdy[f].size())
                                {
                                    cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                    cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                    cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.V_dxdy[f].size()<<endl;
                                    exit(1);
                                }
								call_fftw_convolution(profile.stokes_derivatives.V_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                            }
                        }
                    }
                }
                else // ! derivative_for_gaussian
                {
                    for(int f=0; f < profile.stokes_derivatives.I_dxdy.size(); f++)
                    {
                        if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                        {
                            if (model.n_total_wave !=profile.stokes_derivatives.I_dxdy[f].size())
                            {
                                cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.I_dxdy[f].size()<<endl;
                                exit(1);
                            }
							call_fftw_convolution(profile.stokes_derivatives.I_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                        }
                    }
                    for(int f=0; f < profile.stokes_derivatives.Q_dxdy.size(); f++)
                    {
                        if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                        {
                            if (model.n_total_wave !=profile.stokes_derivatives.Q_dxdy[f].size())
                            {
                                cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.Q_dxdy[f].size()<<endl;
                                exit(1);
                            }
							call_fftw_convolution(profile.stokes_derivatives.Q_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                        }
                    }
                    for(int f=0; f < profile.stokes_derivatives.U_dxdy.size(); f++)
                    {
                        if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") ) 
                        {
                            if (model.n_total_wave !=profile.stokes_derivatives.U_dxdy[f].size())
                            {
                                cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.U_dxdy[f].size()<<endl;
                                exit(1);
                            }
							call_fftw_convolution(profile.stokes_derivatives.U_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                        }
                    }
                    for(int f=0; f < profile.stokes_derivatives.V_dxdy.size(); f++)
                    {
                        if ( (profile.stokes_derivatives.order[f] != "stray_light") && (profile.stokes_derivatives.order[f] != "stray_light_shift") )
                        {
                            if (model.n_total_wave !=profile.stokes_derivatives.V_dxdy[f].size())
                            {
                                cerr<<__FILE__<<":"<<__LINE__<<": The input derivative vectors are not of the same length or mismatch data sizes"<<endl;
                                cerr<<"Size of total number of waves: "<<model.n_total_wave<<endl;
                                cerr<<"Size of vector "<<profile.stokes_derivatives.order[f]<<": "<<profile.stokes_derivatives.V_dxdy[f].size()<<endl;
                                exit(1);
                            }
							call_fftw_convolution(profile.stokes_derivatives.V_dxdy[f], gauss, n, index, forward_plan, backward_plan, in, out);
                        }
                    }
                }
            }
      
            if (derivative_for_gaussian)
            {
				call_fftw_convolution(profile.stokes_derivatives.I_dxdy[index_for_gaussian_derivative], gauss, n, index, forward_plan, backward_plan, in, out);
				call_fftw_convolution(profile.stokes_derivatives.Q_dxdy[index_for_gaussian_derivative], gauss, n, index, forward_plan, backward_plan, in, out);
				call_fftw_convolution(profile.stokes_derivatives.U_dxdy[index_for_gaussian_derivative], gauss, n, index, forward_plan, backward_plan, in, out);
				call_fftw_convolution(profile.stokes_derivatives.V_dxdy[index_for_gaussian_derivative], gauss, n, index, forward_plan, backward_plan, in, out);
            }
      
            // move the index forward for the next region
            index += model.regions[iregion].n_wavelengths;
      
            // clean up
            delete [] gauss;
            if (derivative_for_gaussian)
                delete [] dgauss__dvmac;
			delete [] in;
			delete [] out;
			fftw_destroy_plan(forward_plan);
			fftw_destroy_plan(backward_plan);
        } // end for regions
    }; // end function void convolve_profile(etc.)
  
} // end namespace CSAC
