/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Exception.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_Exception_h_
#define MERLIN_Exception_h_ 1

#include <string>
using std::string;

#include <iostream>
using std::endl;
using std::ostream;

#include "MERLIN_Object.h"
using CSAC::MERLIN_Object;

namespace CSAC {
  
    /// \brief Fundamental and unique type of exception raised by problems encountered in MERLIN.  
    ///
    /// About error codes: \n
    /// Error codes must be a integer value in the set [1,...,255]
    /// values outside this range are converted by the kernel
    /// to values in the range. This is how Unix works. For futher details
    /// read about return values by applications in Unix.
  
    class MERLIN_Exception : public MERLIN_Object
    {
        /// Description of the exception.
        string _what;
        /// Error code for the exception.
        int _error_code;
        /// Internal buffer for conversions of doubles and integers.
        /// You may change the size if it does not fit your application.
        char _buffer[512];
    public:
        /// \brief Default constructor.
        ///
        /// Make an empty MERLIN_Exception object with the error code set to zero.
        MERLIN_Exception() :
            _what(""),
            _error_code(0)
        {}
  
        /// \brief Construct an object of this class with a message and an error code.
        ///
        /// By using this constructor, you know what is your error code and what message
        /// you wish to send along with the exception. You can modify the object further 
        /// by using any of the overloaded operators '+' to further append data to the 
        /// contents of this object.
        MERLIN_Exception(int error, const string & message) 
        {
            _what = message;
            _error_code = error;
        }
        /// Destructor.
        virtual  ~MERLIN_Exception(){}
  
  
  
  
  
        /// Concatenate an MERLIN_Exception object.
        MERLIN_Exception& operator+ (MERLIN_Exception const &s);
        /// Concatenate to this object's message a const char* buffer 's'.
        MERLIN_Exception& operator+ (const char *s);
        /// Concatenate to this object's message an integer value 'i'.
        MERLIN_Exception& operator+ (int i);
        /// Concatenate to this object's message with a double value 'd'.
        MERLIN_Exception& operator+ (double d);
        /// Concatenate to this object's message a character 'c'.
        MERLIN_Exception& operator+ (char c);
        /// Returns a copy of the message in this object
  
  
        /// Get the message that comes along with an exception raised by this object.
        string get_description() {
            return _what;
        }
        /// Set the error code for this object.
        void set_error_code(const int &error){
            _error_code = error;
        }
        /// Get the error code for this object.
        int get_error_code(){
            return _error_code;
        }
  
        /// \brief Required method to implement overloaded inserter operator via inheritance from MERLIN_Object.
        ///
        /// Use this operator to quickly send to a stream the 
        /// contents of this object, for example:
        /** 
            \code
            #include "MERLIN_Exception.h"
            using CSAC::MERLIN_Exception
            try 
            {
            ... // we are doing some work here
            ... // but a problem has ocurred
            MERLIN_Exception some_problem;
            ... // Add or modify the contents of the exception.
            ... // Now throw the exception
            throw some_problem;  
            }
            // catch the exception
            catch(MERLIN_Exception &ex)
            {
            // show the exception in standard output.
            cout<<some_problem;
            // exit using the exception error code
            exit(some_problem.get_error_code());
            }
            \endcode
        */
        virtual void dump( ostream &stream ) const 
        {
            stream << "MERLIN_Exception: " << (void *) this<< ":" << endl ;
            stream << "Error code = " << _error_code << endl;
            stream << "Message = "<< _what << endl;
        }
    };






} // end namespace CSAC

#endif // MERLIN_Exception_h_



