/* ============================================================================
* Language: C++ 
* File: MERLIN_InversionStepWiseRegression.cc
* Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
* Sub-package: Fundamental Inversion Code (FICO).
*
* Created by Jose Garcia <jgarcia@ucar.edu> on 05/15/07.
*  
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* You can contact University Corporation for Atmospheric Research at
* 3080 Center Green Drive, Boulder, CO 80301

* (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
* Please read the full copyright statement in the file COPYRIGHT_UCAR.
*
=========================================================================== */
#include <cstdio>

// for DBL_MAX and DBL_MIN
#include <float.h>

#include "MERLIN_InversionStepWiseRegression.h"
#include "MERLIN_Chisq.h"
#include "MERLIN_PrintFormated.h"
#include "MERLIN_SweepOperator.h"


using CSAC::MERLIN_InversionStepWiseRegression;
using CSAC::MERLIN_MilneEddington;
using CSAC::MERLIN_StokesProfile;
using CSAC::MERLIN_ParametersLimits;
using CSAC::MERLIN_ModelCorrection;
using CSAC::MERLIN_Chisq;
using CSAC::MERLIN_SweepOperator;
using CSAC::STOKES_I_WEIGHT;
using CSAC::STOKES_Q_WEIGHT;
using CSAC::STOKES_U_WEIGHT;
using CSAC::STOKES_V_WEIGHT;

void build_complete_matrix(MERLIN_Model &trial, 
                           MERLIN_StokesProfile &observed,
                           MERLIN_Profile &synthetic, 
                           double **A, 
                           int dimension, 
                           double &chisq)
{
    if (CSAC::merlin_debug)
    {
        cout<<"Stokes I weight is "<<STOKES_I_WEIGHT<<endl;
        cout<<"Stokes Q weight is "<<STOKES_Q_WEIGHT<<endl;
        cout<<"Stokes U weight is "<<STOKES_U_WEIGHT<<endl;
        cout<<"Stokes V weight is "<<STOKES_V_WEIGHT<<endl;
    }
    if (trial.vectors_to_invert.invert_I && trial.vectors_to_invert.invert_Q && trial.vectors_to_invert.invert_U && trial.vectors_to_invert.invert_V)
    {
        for (int i_param=0; i_param < dimension - 1; i_param++)
        {
            for (int j_param=0; j_param < dimension - 1; j_param++)
            {
                A[i_param][j_param]=0.;
                for (int i_data=0; i_data < trial.n_total_wave; i_data++)
                {
                    A[i_param][j_param] = A[i_param][j_param] +
                    synthetic.stokes_derivatives.I_dxdy[i_param][i_data] * synthetic.stokes_derivatives.I_dxdy[j_param][i_data] * STOKES_I_WEIGHT+
                    synthetic.stokes_derivatives.Q_dxdy[i_param][i_data] * synthetic.stokes_derivatives.Q_dxdy[j_param][i_data] * STOKES_Q_WEIGHT+
                    synthetic.stokes_derivatives.U_dxdy[i_param][i_data] * synthetic.stokes_derivatives.U_dxdy[j_param][i_data] * STOKES_U_WEIGHT+
                    synthetic.stokes_derivatives.V_dxdy[i_param][i_data] * synthetic.stokes_derivatives.V_dxdy[j_param][i_data] * STOKES_V_WEIGHT;
                }
            }
            A[i_param][dimension - 1] = 0.;
            for (int i_data=0; i_data < trial.n_total_wave; i_data++)
            {
                A[i_param][dimension - 1] = A[i_param][dimension - 1] +  
                (observed.I[i_data]-synthetic.stokes.I[i_data]) * synthetic.stokes_derivatives.I_dxdy[i_param][i_data] * STOKES_I_WEIGHT+
                (observed.Q[i_data]-synthetic.stokes.Q[i_data]) * synthetic.stokes_derivatives.Q_dxdy[i_param][i_data] * STOKES_Q_WEIGHT+
                (observed.U[i_data]-synthetic.stokes.U[i_data]) * synthetic.stokes_derivatives.U_dxdy[i_param][i_data] * STOKES_U_WEIGHT+
                (observed.V[i_data]-synthetic.stokes.V[i_data]) * synthetic.stokes_derivatives.V_dxdy[i_param][i_data] * STOKES_V_WEIGHT;
            }
        }
        
        for (int j_param=0; j_param < dimension - 1; j_param++)
        {
            A[dimension - 1][j_param] = A[j_param][dimension - 1];
        }
        A[dimension - 1][dimension - 1] = chisq;
    }
    else 
    {
        char error_line[32];
        snprintf(error_line,sizeof(error_line), "%i", __LINE__);
        string t_error=__FILE__ + string(":") + string(error_line) + ": STOP! Exception caught. This class can not manage the case when some of the Stokes vectors are not to be inverted";
        throw t_error;
    }
    
}

double 
MERLIN_InversionStepWiseRegression::execute (
                                             MERLIN_StokesProfile &observed_profile, 
                                             MERLIN_Model &guess, 
                                             MERLIN_StokesProfile *sigma, 
                                             int &n_iterations,
                                             double &chisq_I,
                                             double &chisq_U,
                                             double &chisq_Q,
                                             double &chisq_V
                                             )
{
    
    _why_did_we_stop = UNSET_INVERSION_NOT_PERFORMED;
    
    MERLIN_Model trial;
    trial=guess;
    MERLIN_ModelCorrection::correct_trial_model(trial, limits_for_inversion); 
    
    if (CSAC::merlin_debug)
    {
        cout<<"executing inversion, starting with initial guess:"<<endl;
        cout<<trial;
    }
    
    MERLIN_MilneEddington f;
    MERLIN_Profile trial_profile;

    if (CSAC::merlin_debug)
    {
        cout<<"Calling Milne-Eddington Model for profile and derivatives"<<endl;
    }
    
    // SAG: Compute chi^2 and return an MERLIN_Chisq object.
    //      From the returned object, both chi^2 and the scale 
    //      factor for chi^2 can be extracted. The scale factor
    //      is needed for the uncertainty calculation, because
    //      chi^2 and the curvature matrix are not computed with
    //      consistent scales.
    //
    f.get_profile_and_derivatives(trial, trial_profile);
    MERLIN_Chisq chisq_res; 
    chisq_res = MERLIN_Chisq::compute_chisq(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
    
    // SAG: Extract results
    double chisq        = chisq_res.get_chisq();
    double scale_factor = chisq_res.get_scale_factor();     
    
    // get sure the observed profile is equal in size to the synthetic one
    if( observed_profile.I.size() != trial_profile.stokes.I.size() )
    {
        char observed_size[32];
        snprintf(observed_size, sizeof(observed_size), "%i", observed_profile.I.size());
        char synthetic_size[32];
        snprintf(synthetic_size, sizeof(synthetic_size), "%i", trial_profile.stokes.I.size());
        string error = string("The size of the observed vector (") + string(observed_size) +string(") does not match the size of the synthetic vector (") + string (synthetic_size) + string(")");
        throw error;
    } 
    // get sure both observed and synthetic are at the same wavelength
    for(int h = 0; h < observed_profile.I.size(); h++)
    {
        if ( observed_profile.lambda[h] != trial_profile.stokes.lambda[h] )
        {
            char error_line[32];
            snprintf(error_line,sizeof(error_line), "%i", __LINE__);
            string t_error=__FILE__ + string(":") + string(error_line) + ": STOP! Exception caught.\nThe observed profile and the theoretical profile does not match in wavelength.\n";
            char number_buf[64];
            snprintf(number_buf, sizeof(number_buf), "%i",h);
            t_error += string("At position ") + number_buf + string(" (counting from 0) on grid.\n");
            snprintf(number_buf, sizeof(number_buf), "%.16f",observed_profile.lambda[h]);
            t_error += string("Observed wavelength = ") + number_buf + string("\n");
            snprintf(number_buf, sizeof(number_buf), "%.16f",trial_profile.stokes.lambda[h]);
            t_error += string("Synthetic wavelength = ") + number_buf + string("\n");
            throw t_error;
        }
    }
    
    // Allocate space for the solution matrix and a copy
    int total_matrix_size = guess.n_total_free_parameters + 1;
    double **A = new double* [total_matrix_size]; 
    double **A_copy = new double* [total_matrix_size]; 
    for (int k = 0; k < total_matrix_size ; k++)
    {
        A[k] = new double[total_matrix_size];
        A_copy[k] = new double[total_matrix_size];
    }
    
    
    double cappa = 100;
    double new_chisq;
    
    n_iterations = 0;
    
    for (int r =0; r < _max_iters; r++)
    {
        n_iterations++;
        build_complete_matrix(trial, observed_profile, trial_profile, A, total_matrix_size, chisq);
        
        for (int k = 0; k < total_matrix_size ; k++)
        {
            memcpy(A_copy[k],A[k], (total_matrix_size * sizeof(double)));
        }
        
	int *used;
        used = new int [total_matrix_size];
        MERLIN_SweepOperator::sweep(A_copy, used, total_matrix_size ,total_matrix_size-1);
        zero_dependent_variables(A_copy, used, total_matrix_size);
        
        if(CSAC::merlin_debug)
        {
            cout.precision(15);
            for(int i=0; i<total_matrix_size; i++)
            {
                std::cout<<'[';
                for (int j=0; j< total_matrix_size; j++)
                {
                    CSAC::MERLIN_PrintFormated::print_blocked(std::cout,A[i][j]);
                }
                std::cout<<']'<<std::endl;
            }
            cout<<"----------------------------------------------------------\n"<<endl;
            cout<<'\n';
            for(int y=0; y < total_matrix_size; y++)
                cout<<used[y]<<' ';
            cout<<endl;
        }  
        
        for (int i = 0; i < total_matrix_size -1; i++)
        {
            if (A_copy[i][i] >= 0.0)
            {
                for (int j = 0; j < total_matrix_size -1; j++)
                {
                    A[j][i] = 0.0;
                    A[i][j] = 0.0;
                }
            }
        }
        
        for (int k = 0; k < total_matrix_size ; k++)
        {
            memcpy(A_copy[k], A[k], (total_matrix_size * sizeof(double)));
            A_copy[k][k] *= (1 +cappa);
        }
        
        MERLIN_SweepOperator::sweep(A_copy, used, total_matrix_size ,total_matrix_size-1);
        zero_dependent_variables(A_copy, used, total_matrix_size);
        
        if(CSAC::merlin_debug)
        {
            cout.precision(15);
            for(int i=0; i<total_matrix_size; i++)
            {
                std::cout<<'[';
                for (int j=0; j< total_matrix_size; j++)
                {
                    CSAC::MERLIN_PrintFormated::print_blocked(std::cout,A_copy[i][j]);
                }
                std::cout<<']'<<std::endl;
            }
        } 
        
        for (int p=0; p < trial_profile.stokes_derivatives.order.size(); p++)
        {
            if (CSAC::merlin_debug) 
            {
                cout<<"Correcting "<<trial_profile.stokes_derivatives.order[p]<<" by applying value "<<A_copy[p][total_matrix_size -1]<<endl;
            }
            if (trial_profile.stokes_derivatives.order[p]=="B0") trial.planck_function.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="B1") trial.planck_function_gradient.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="B") trial.magnetic_field.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="Eta_0") trial.line_strength.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="Inclination") trial.inclination.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="Azimuth") trial.azimuth.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="Dldop") trial.doppler_width.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="a") trial.damping.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="vmac") trial.vmac.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="stray_light") trial.stray_light.value += A_copy[p][total_matrix_size -1];
            else if (trial_profile.stokes_derivatives.order[p]=="stray_light_shift") trial.stray_light_shift.value += A_copy[p][total_matrix_size -1];
            else
            {
                char buffer[30];
                int h=0;
                for(int iline=0; iline < trial.n_lines; iline++)
                {
                    if (trial.lines[iline].wavelength_shift.free)
                    {
                        sprintf(buffer, "%i", h); 
                        string line_name = string("v") + buffer;
                        if (trial_profile.stokes_derivatives.order[p]==line_name)
                            trial.lines[iline].wavelength_shift.value += A_copy[p][total_matrix_size -1];
                        h++;
                    }
                }
            }
        }
        // Uncomment these lines to see the parameter object as the inversion moves forward
        if (CSAC::merlin_debug)
            cout<<"%%%%%%%%%%%%% Before correction %%%%%%%%%%%%"<<endl<<trial;
        MERLIN_ModelCorrection::correct_trial_model(trial, limits_for_inversion); 
        if (CSAC::merlin_debug)
            cout<<"%%%%%%%%%%%%% After correction %%%%%%%%%%%%"<<endl<<trial;
        
        f.get_profile_and_derivatives(trial, trial_profile);
        new_chisq = MERLIN_Chisq::compute_chisq(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
        if (new_chisq < chisq)
        {
            guess = trial;
            cappa *= (1E-1);
            chisq = new_chisq;
            if(CSAC::merlin_debug)
                cout<<"Accepted new model, chisq = "<<chisq<<" lambda = "<<cappa<<endl;
        }
        else
        {
            trial = guess;
            f.get_profile_and_derivatives(trial, trial_profile);
            chisq = MERLIN_Chisq::compute_chisq(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
            //cappa *= 10;
			// don't multiply with the same factor as we divide by.
            cappa *= 8;
            if(CSAC::merlin_debug)
                cout<<"Rejected new model, chisq = "<<chisq<<" lambda = "<<cappa<<endl;
        }
        if (cappa < _min_damping_parameter)
        {
            _why_did_we_stop = LAMBDA_SMALLER_THAM_MIN_ALLOWED;
            break;
        }
        if (cappa > _max_damping_parameter)
        {
            _why_did_we_stop = LAMBDA_GREATER_THAM_MAX_ALLOWED;
            break;
        }
        
		delete [] used;
    }
    
    if (n_iterations == _max_iters)
        _why_did_we_stop = REACHED_MAX_ITERATIONS;
    
    if(CSAC::merlin_debug)
    {
        cout<<"Returning with chisq = "<<chisq<<endl;
    }
    
    chisq_I =  MERLIN_Chisq::compute_chisq_I(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
    chisq_Q =  MERLIN_Chisq::compute_chisq_Q(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
    chisq_U =  MERLIN_Chisq::compute_chisq_U(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
    chisq_V =  MERLIN_Chisq::compute_chisq_V(observed_profile, trial_profile.stokes, guess, sigma, _continuum_normalization);
    

    // Compute the errors for the inversion parameters
    trial = guess;
    f.get_profile_and_derivatives(trial, trial_profile);
    cappa = (1.0);
    //build_complete_matrix(trial, observed_profile, trial_profile, A, total_matrix_size, chisq);

    // SAG: Compute the parameter uncertainty. The uncertainties are
    //      placed in the guess object
    compute_uncertainty(guess,trial,observed_profile,trial_profile,total_matrix_size,chisq,scale_factor);
    
    // release memory...
    for (int k = 0; k < total_matrix_size ; k++)
    {
        delete [] A_copy[k];
        delete [] A[k];
        A_copy[k] = 0;
        A[k] = 0;
    }
    delete [] A_copy;
    delete [] A;
        
    return chisq;
}

// SAG: Compute the uncertainty in the fitting parameters
void MERLIN_InversionStepWiseRegression :: compute_uncertainty(
                                                       MERLIN_Model &guess, 
                                                       MERLIN_Model &trial,     
                                                       MERLIN_StokesProfile &observed_profile,
                                                       MERLIN_Profile &trial_profile, 
                                                       int total_matrix_size, 
                                                       double chisq,
                                                       double scale_factor
                                                       )
{
 
    // SAG: Allocate memory for parameter uncertainties
    double *uncertainty = new double[total_matrix_size];  
    
    // SAG: Allocate memory for the covariance matrix
    double **covariance_matrix = new double* [total_matrix_size];
    for(int i = 0; i < total_matrix_size; i++){
        covariance_matrix[i] = new double[total_matrix_size];  
    }
        
    // SAG: Construct curvature matrix and place it in covariance
    // matrix
    //
    build_complete_matrix(trial, observed_profile, trial_profile,covariance_matrix, total_matrix_size, chisq);
  
    // SAG: Invert the curvature matrix. 
    // 
    // Note that, given the way that
    // the sweep operator is implemented, sweep actually returns the negative
    // of the inverse. The calculation is inplace, so after the call to sweep, 
    // the elements A[0:total_matrix_size-2][0:total_matrix_size-2] is
    // the negative of the covariance matrix. 
    //
    // Also note that, since sweep is called with the last argument 
    // set to total_matrix_size-1, used[total_matrix_size-1] == 0, always.
    //
    int *used;
    used = new int [total_matrix_size];
    MERLIN_SweepOperator::sweep(covariance_matrix, used, total_matrix_size ,total_matrix_size-1);

    // SAG: Determine number of fitting parameters
    unsigned int N_params = 0;
    for(int i=0;i < total_matrix_size; i++){
        if(used[i] == 1){
            N_params++;  
        }
    } 
    
    // SAG: Compute uncertainty
    // See Equation (18) of Sanchez Almeida, J.: ApJ,491.993-1008,DOI: 10.1086/304999
    // If parameter is not used, then uncertainty is zero. 
    //
    // Note that because of the way that the sweep operator is implemented
    // the diagonals of the covariance matrix are always negative.
    //
    // Also note that this calculation is inplace, so the diagonal
    // of the covariance matrix is overwritten with the uncertainty
    // in fit parameter i
    //
    double arg_of_sqrt;
    for(int i=0; i < total_matrix_size; i++){
        if(used[i] == 1){
       
            // Compute the argument of the square root. This should
            // be negative, due to the way A[i][i] is computed.
            //
            arg_of_sqrt    = scale_factor*chisq*covariance_matrix[i][i]/N_params;
            uncertainty[i] = sqrt(-arg_of_sqrt);
        
         // If used[i] is not set to one, then set uncertainty to zero.
         // This occurs when a parameter is fixed and is not part of the
         // fitting
         } else {
             uncertainty[i] = 0;      
         }
    }
    delete [] used;
    
    // SAG: Write uncertainty to the profile
    int index = 0;
    for (int p=0; p < trial_profile.stokes_derivatives.order.size(); p++)
        {
            if (trial_profile.stokes_derivatives.order[p]=="B0") 
                guess.planck_function.error = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="B1") 
                guess.planck_function_gradient.error = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="B") 
                guess.magnetic_field.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="Eta_0") 
                guess.line_strength.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="Inclination") 
                guess.inclination.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="Azimuth") 
                guess.azimuth.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="Dldop") 
                guess.doppler_width.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="a") 
                guess.damping.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="vmac") 
                guess.vmac.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="stray_light") 
                guess.stray_light.error  = uncertainty[index];
            else if (trial_profile.stokes_derivatives.order[p]=="stray_light_shift") 
                guess.stray_light_shift.error  = uncertainty[index];
            else
            {
                char buffer[30];
                int h=0;
                for(int iline=0; iline < guess.n_lines; iline++)
                {
                    if (guess.lines[iline].wavelength_shift.free)
                    {
                        sprintf(buffer, "%i", h); 
                        string line_name = string("v") + buffer;
                        if (trial_profile.stokes_derivatives.order[p]==line_name)
                            guess.lines[iline].wavelength_shift.error  = uncertainty[index];;
                        h++;
                    }
                }
            }
            index++;
        }    

    // SAG: Free local memory
    for(int i = 0; i < total_matrix_size; i++){
        delete [] covariance_matrix[i]; 
    }
    delete [] covariance_matrix;
    delete [] uncertainty;
}