/* ============================================================================
 * Language: C++ 
 * File: MERLIN_MilneEddington.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>

#include <string>
using std::string ;

#include <iostream>
using std::cerr ;
using std::cout ;
using std::endl ;


#define MINIMUM_LINE_PROXIMITY 5.0


// Local imports from MERLIN package
#include "MERLIN.h"
#include "MERLIN_CommonFactors.h"
#include "MERLIN_MVoigt.h"
#include "MERLIN_ConvolveProfile.h"
#include "MERLIN_MilneEddington.h"
using CSAC::MERLIN_MilneEddington;



/////////////////////////////////////////////////////////////
//
// Derivatives of I with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dI__dB1()
{
	// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
	return  (Eta_I * common_factor12) / Delta;
}

inline double MERLIN_MilneEddington::compute_dI__dB()
{
	return  
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta *
		(
		 common_factor12 * dEta_I__dB 
		 + 
		 2 * ( Eta_I*dEta_I__dB + Rho_Q*dRho_Q__dB +  Rho_U*dRho_U__dB + Rho_V*dRho_V__dB ) * Eta_I
		) 
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dB);
}

inline double MERLIN_MilneEddington::compute_dI__dEta_0()
{
	return  
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta *
		(
		 common_factor12 * dEta_I__dEta_0 
		 + 
		 2 * ( Eta_I*dEta_I__dEta_0 + Rho_Q*dRho_Q__dEta_0 +  Rho_U*dRho_U__dEta_0 + Rho_V*dRho_V__dEta_0 ) * Eta_I
		)
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dEta_0);
}


inline double MERLIN_MilneEddington::compute_dI__dDldop()
{
	return 
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta *
		(
		 common_factor12 * dEta_I__dDldop 
		 + 
		 2 * ( Eta_I*dEta_I__dDldop + Rho_Q*dRho_Q__dDldop +  Rho_U*dRho_U__dDldop + Rho_V*dRho_V__dDldop ) * Eta_I
		)
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dDldop);
}


inline double MERLIN_MilneEddington::compute_dI__dInclination()
{
	return 
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta * 
		(
		 common_factor12 * dEta_I__dInclination + 
		 2 * (Eta_I*dEta_I__dInclination + Rho_Q*dRho_Q__dInclination +  Rho_U*dRho_U__dInclination + Rho_V*dRho_V__dInclination ) * Eta_I
		)  
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dInclination);
}

inline double MERLIN_MilneEddington::compute_dI__da()
{
	return 
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta *
		(
		 common_factor12 * dEta_I__da 
		 + 
		 2 * ( Eta_I*dEta_I__da + Rho_Q*dRho_Q__da +  Rho_U*dRho_U__da + Rho_V*dRho_V__da ) * Eta_I
		) 
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__da);
}


inline double MERLIN_MilneEddington::compute_dI__dAzimuth()
{
	return 
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta * 
		(
		 2 * ( Rho_Q*dRho_Q__dAzimuth +  Rho_U*dRho_U__dAzimuth ) * Eta_I 
		)
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dAzimuth);
}


inline double MERLIN_MilneEddington::compute_dI__dv(int &j)
{
	return
		// remember common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
		B1_divided_by_Delta * 
		(
		 common_factor12 * dEta_I__dv[j] 
		 + 
		 2 * ( Eta_I * dEta_I__dv[j] + Rho_Q*dRho_Q__dv[j] +  Rho_U*dRho_U__dv[j] + Rho_V*dRho_V__dv[j] ) * Eta_I
		) 
		-
		(B1_divided_by_Delta_square * common_factor12 * Eta_I * dDelta__dv[j]);
}

/////////////////////////////////////////////////////////////
//
// Derivatives of Q with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dQ__dB1()
{
	return  (-1/Delta) * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3);
}

inline double MERLIN_MilneEddington::compute_dQ__dB()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dB * Eta_Q + Eta_I_square * dEta_Q__dB) +
		 (dEta_I__dB * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I * (dEta_V__dB * Rho_U + Eta_V * dRho_U__dB - dEta_U__dB * Rho_V - Eta_U * dRho_V__dB)) +
		 (dRho_Q__dB * det3 + Rho_Q * (dEta_Q__dB * Rho_Q + Eta_Q * dRho_Q__dB + dEta_U__dB * Rho_U + Eta_U * dRho_U__dB + dEta_V__dB * Rho_V + Eta_V * dRho_V__dB))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dB; 
}

inline double MERLIN_MilneEddington::compute_dQ__dEta_0()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dEta_0 * Eta_Q + Eta_I_square * dEta_Q__dEta_0) +
		 (dEta_I__dEta_0 * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I * (dEta_V__dEta_0 * Rho_U + Eta_V * dRho_U__dEta_0 - dEta_U__dEta_0 * Rho_V - Eta_U * dRho_V__dEta_0)) +
		 (dRho_Q__dEta_0 * det3 + Rho_Q * (dEta_Q__dEta_0 * Rho_Q + Eta_Q * dRho_Q__dEta_0 + dEta_U__dEta_0 * Rho_U + Eta_U * dRho_U__dEta_0 + dEta_V__dEta_0 * Rho_V + Eta_V * dRho_V__dEta_0))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dEta_0;
}


inline double MERLIN_MilneEddington::compute_dQ__dDldop()
{
	return 
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dDldop * Eta_Q + Eta_I_square * dEta_Q__dDldop) +
		 (dEta_I__dDldop * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I * (dEta_V__dDldop * Rho_U + Eta_V * dRho_U__dDldop - dEta_U__dDldop * Rho_V - Eta_U * dRho_V__dDldop)) +
		 (dRho_Q__dDldop * det3 + Rho_Q * (dEta_Q__dDldop * Rho_Q + Eta_Q * dRho_Q__dDldop + dEta_U__dDldop * Rho_U + Eta_U * dRho_U__dDldop + dEta_V__dDldop * Rho_V + Eta_V * dRho_V__dDldop))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dDldop;
}


inline double MERLIN_MilneEddington::compute_dQ__dInclination()
{
	return 
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dInclination * Eta_Q + Eta_I_square * dEta_Q__dInclination) +
		 (dEta_I__dInclination * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I * (dEta_V__dInclination * Rho_U + Eta_V * dRho_U__dInclination - dEta_U__dInclination * Rho_V - Eta_U * dRho_V__dInclination)) +
		 (dRho_Q__dInclination * det3 + Rho_Q * (dEta_Q__dInclination * Rho_Q + Eta_Q * dRho_Q__dInclination + dEta_U__dInclination * Rho_U + Eta_U * dRho_U__dInclination + dEta_V__dInclination * Rho_V + Eta_V * dRho_V__dInclination))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dInclination;
}

inline double MERLIN_MilneEddington::compute_dQ__da()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__da * Eta_Q + Eta_I_square * dEta_Q__da) +
		 (dEta_I__da * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I * (dEta_V__da * Rho_U + Eta_V * dRho_U__da - dEta_U__da * Rho_V - Eta_U * dRho_V__da)) +
		 (dRho_Q__da * det3 + Rho_Q * (dEta_Q__da * Rho_Q + Eta_Q * dRho_Q__da + dEta_U__da * Rho_U + Eta_U * dRho_U__da + dEta_V__da * Rho_V + Eta_V * dRho_V__da))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__da; 
}


inline double MERLIN_MilneEddington::compute_dQ__dAzimuth()
{
	return  
		-B1_divided_by_Delta *
		(
		 (Eta_I_square * dEta_Q__dAzimuth) +
		 (Eta_I * (Eta_V * dRho_U__dAzimuth - dEta_U__dAzimuth * Rho_V)) +
		 (dRho_Q__dAzimuth * det3 + Rho_Q * (dEta_Q__dAzimuth * Rho_Q + Eta_Q * dRho_Q__dAzimuth + dEta_U__dAzimuth * Rho_U + Eta_U * dRho_U__dAzimuth))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dAzimuth; 
}


inline double MERLIN_MilneEddington::compute_dQ__dv(int &j)
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dv[j] * Eta_Q + Eta_I_square * dEta_Q__dv[j]) +
		 (dEta_I__dv[j] * (Eta_V * Rho_U - Eta_U * Rho_V) + Eta_I *(dEta_V__dv[j] * Rho_U + Eta_V * dRho_U__dv[j] - dEta_U__dv[j] * Rho_V - Eta_U * dRho_V__dv[j])) +
		 (dRho_Q__dv[j] * det3 + Rho_Q * (dEta_Q__dv[j] * Rho_Q + Eta_Q * dRho_Q__dv[j] + dEta_U__dv[j] * Rho_U + Eta_U * dRho_U__dv[j] + dEta_V__dv[j] * Rho_V + Eta_V * dRho_V__dv[j]))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) * dDelta__dv[j]; 	 
}


/////////////////////////////////////////////////////////////
//
// Derivatives of U with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dU__dB1()
{
	return  (-1/Delta) * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3);
}

inline double MERLIN_MilneEddington::compute_dU__dB()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dB * Eta_U + Eta_I_square * dEta_U__dB) +
		 (dEta_I__dB * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__dB * Rho_V + Eta_Q * dRho_V__dB - dEta_V__dB * Rho_Q - Eta_V * dRho_Q__dB)) +
		 (dRho_U__dB * det3 + Rho_U * (dEta_Q__dB * Rho_Q + Eta_Q * dRho_Q__dB + dEta_U__dB * Rho_U + Eta_U * dRho_U__dB + dEta_V__dB * Rho_V + Eta_V * dRho_V__dB))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dB; 
}

inline double MERLIN_MilneEddington::compute_dU__dEta_0()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dEta_0 * Eta_U + Eta_I_square * dEta_U__dEta_0) +
		 (dEta_I__dEta_0 * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__dEta_0 * Rho_V + Eta_Q * dRho_V__dEta_0 - dEta_V__dEta_0 * Rho_Q - Eta_V * dRho_Q__dEta_0)) +
		 (dRho_U__dEta_0 * det3 + Rho_U * (dEta_Q__dEta_0 * Rho_Q + Eta_Q * dRho_Q__dEta_0 + dEta_U__dEta_0 * Rho_U + Eta_U * dRho_U__dEta_0 + dEta_V__dEta_0 * Rho_V + Eta_V * dRho_V__dEta_0))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dEta_0; 
}

inline double MERLIN_MilneEddington::compute_dU__dDldop()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dDldop * Eta_U + Eta_I_square * dEta_U__dDldop) +
		 (dEta_I__dDldop * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__dDldop * Rho_V + Eta_Q * dRho_V__dDldop - dEta_V__dDldop * Rho_Q - Eta_V * dRho_Q__dDldop)) +
		 (dRho_U__dDldop * det3 + Rho_U * (dEta_Q__dDldop * Rho_Q + Eta_Q * dRho_Q__dDldop + dEta_U__dDldop * Rho_U + Eta_U * dRho_U__dDldop + dEta_V__dDldop * Rho_V + Eta_V * dRho_V__dDldop))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dDldop; 
}

inline double MERLIN_MilneEddington::compute_dU__dInclination()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dInclination * Eta_U + Eta_I_square * dEta_U__dInclination) +
		 (dEta_I__dInclination * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__dInclination * Rho_V + Eta_Q * dRho_V__dInclination - dEta_V__dInclination * Rho_Q - Eta_V * dRho_Q__dInclination)) +
		 (dRho_U__dInclination * det3 + Rho_U * (dEta_Q__dInclination * Rho_Q + Eta_Q * dRho_Q__dInclination + dEta_U__dInclination * Rho_U + Eta_U * dRho_U__dInclination + dEta_V__dInclination * Rho_V + Eta_V * dRho_V__dInclination))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dInclination; 
}

inline double MERLIN_MilneEddington::compute_dU__da()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__da * Eta_U + Eta_I_square * dEta_U__da) +
		 (dEta_I__da * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__da * Rho_V + Eta_Q * dRho_V__da - dEta_V__da * Rho_Q - Eta_V * dRho_Q__da)) +
		 (dRho_U__da * det3 + Rho_U * (dEta_Q__da * Rho_Q + Eta_Q * dRho_Q__da + dEta_U__da * Rho_U + Eta_U * dRho_U__da + dEta_V__da * Rho_V + Eta_V * dRho_V__da))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__da; 
}

inline double MERLIN_MilneEddington::compute_dU__dAzimuth()
{
	return  
		-B1_divided_by_Delta *
		(
		 (Eta_I_square * dEta_U__dAzimuth) +
		 (Eta_I * (dEta_Q__dAzimuth * Rho_V - Eta_V * dRho_Q__dAzimuth)) +
		 (dRho_U__dAzimuth * det3 + Rho_U * (dEta_Q__dAzimuth * Rho_Q + Eta_Q * dRho_Q__dAzimuth + dEta_U__dAzimuth * Rho_U + Eta_U * dRho_U__dAzimuth))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dAzimuth; 
}

inline double MERLIN_MilneEddington::compute_dU__dv(int &j)
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dv[j] * Eta_U + Eta_I_square * dEta_U__dv[j]) +
		 (dEta_I__dv[j] * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Eta_I * (dEta_Q__dv[j] * Rho_V + Eta_Q * dRho_V__dv[j] - dEta_V__dv[j] * Rho_Q - Eta_V * dRho_Q__dv[j])) +
		 (dRho_U__dv[j] * det3 + Rho_U * (dEta_Q__dv[j] * Rho_Q + Eta_Q * dRho_Q__dv[j] + dEta_U__dv[j] * Rho_U + Eta_U * dRho_U__dv[j] + dEta_V__dv[j] * Rho_V + Eta_V * dRho_V__dv[j]))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) * dDelta__dv[j]; 
}

/////////////////////////////////////////////////////////////
//
// Derivatives of V with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dV__dB1()
{
	return  (-1/Delta) * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3);
}

inline double MERLIN_MilneEddington::compute_dV__dB()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dB * Eta_V + Eta_I_square * dEta_V__dB) +
		 (dEta_I__dB * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__dB * Rho_Q + Eta_U * dRho_Q__dB - dEta_Q__dB * Rho_U - Eta_Q * dRho_U__dB)) +
		 (dRho_V__dB * det3 + Rho_V * (dEta_Q__dB * Rho_Q + Eta_Q * dRho_Q__dB + dEta_U__dB * Rho_U + Eta_U * dRho_U__dB + dEta_V__dB * Rho_V + Eta_V * dRho_V__dB))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dB; 
}

inline double MERLIN_MilneEddington::compute_dV__dEta_0()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dEta_0 * Eta_V + Eta_I_square * dEta_V__dEta_0) +
		 (dEta_I__dEta_0 * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__dEta_0 * Rho_Q + Eta_U * dRho_Q__dEta_0 - dEta_Q__dEta_0 * Rho_U - Eta_Q * dRho_U__dEta_0)) +
		 (dRho_V__dEta_0 * det3 + Rho_V * (dEta_Q__dEta_0 * Rho_Q + Eta_Q * dRho_Q__dEta_0 + dEta_U__dEta_0 * Rho_U + Eta_U * dRho_U__dEta_0 + dEta_V__dEta_0 * Rho_V + Eta_V * dRho_V__dEta_0))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dEta_0; 
}

inline double MERLIN_MilneEddington::compute_dV__dDldop()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dDldop * Eta_V + Eta_I_square * dEta_V__dDldop) +
		 (dEta_I__dDldop * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__dDldop * Rho_Q + Eta_U * dRho_Q__dDldop - dEta_Q__dDldop * Rho_U - Eta_Q * dRho_U__dDldop)) +
		 (dRho_V__dDldop * det3 + Rho_V * (dEta_Q__dDldop * Rho_Q + Eta_Q * dRho_Q__dDldop + dEta_U__dDldop * Rho_U + Eta_U * dRho_U__dDldop + dEta_V__dDldop * Rho_V + Eta_V * dRho_V__dDldop))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dDldop; 
}

inline double MERLIN_MilneEddington::compute_dV__dInclination()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dInclination * Eta_V + Eta_I_square * dEta_V__dInclination) +
		 (dEta_I__dInclination * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__dInclination * Rho_Q + Eta_U * dRho_Q__dInclination - dEta_Q__dInclination * Rho_U - Eta_Q * dRho_U__dInclination)) +
		 (dRho_V__dInclination * det3 + Rho_V * (dEta_Q__dInclination * Rho_Q + Eta_Q * dRho_Q__dInclination + dEta_U__dInclination * Rho_U + Eta_U * dRho_U__dInclination + dEta_V__dInclination * Rho_V + Eta_V * dRho_V__dInclination))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dInclination; 
}

inline double MERLIN_MilneEddington::compute_dV__da()
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__da * Eta_V + Eta_I_square * dEta_V__da) +
		 (dEta_I__da * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__da * Rho_Q + Eta_U * dRho_Q__da - dEta_Q__da * Rho_U - Eta_Q * dRho_U__da)) +
		 (dRho_V__da * det3 + Rho_V * (dEta_Q__da * Rho_Q + Eta_Q * dRho_Q__da + dEta_U__da * Rho_U + Eta_U * dRho_U__da + dEta_V__da * Rho_V + Eta_V * dRho_V__da))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__da; 
}

inline double MERLIN_MilneEddington::compute_dV__dAzimuth()
{
	return  
		-B1_divided_by_Delta *
		(
		 (Eta_I * (dEta_U__dAzimuth * Rho_Q + Eta_U * dRho_Q__dAzimuth - dEta_Q__dAzimuth * Rho_U - Eta_Q * dRho_U__dAzimuth)) +
		 (Rho_V * (dEta_Q__dAzimuth * Rho_Q + Eta_Q * dRho_Q__dAzimuth + dEta_U__dAzimuth * Rho_U + Eta_U * dRho_U__dAzimuth))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dAzimuth; 
}

inline double MERLIN_MilneEddington::compute_dV__dv(int &j)
{
	return  
		-B1_divided_by_Delta *
		(
		 (2 * Eta_I * dEta_I__dv[j] * Eta_V + Eta_I_square * dEta_V__dv[j]) +
		 (dEta_I__dv[j] * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Eta_I * (dEta_U__dv[j] * Rho_Q + Eta_U * dRho_Q__dv[j] - dEta_Q__dv[j] * Rho_U - Eta_Q * dRho_U__dv[j])) +
		 (dRho_V__dv[j] * det3 + Rho_V * (dEta_Q__dv[j] * Rho_Q + Eta_Q * dRho_Q__dv[j] + dEta_U__dv[j] * Rho_U + Eta_U * dRho_U__dv[j] + dEta_V__dv[j] * Rho_V + Eta_V * dRho_V__dv[j]))
		) +
		B1_divided_by_Delta_square * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) * dDelta__dv[j]; 
}

/////////////////////////////////////////////////////////////
//
// Derivatives of Rho_V with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dRho_V__dInclination()
{
	return 
		(-1) * common_factor0 * // remember common_factor0 = Eta_0_r * .5 
		Sin_inc * common_factor11; // remember common_factor11 = (Psi_r - Psi_b)

}

inline double MERLIN_MilneEddington::compute_dRho_V__dEta_0()
{
	return 
		.5 * Cos_inc * 
		common_factor11 * dEta_0_r__dEta_0; // remember common_factor11 = (Psi_r - Psi_b)
}

inline double MERLIN_MilneEddington::compute_dRho_V__dB()
{
	return 
		common_factor0 *  Cos_inc * // remember common_factor0 = Eta_0_r * .5 
		(dPsi_r__dB - dPsi_b__dB);
}

inline double MERLIN_MilneEddington::compute_dRho_V__dDldop()
{
	return 
		common_factor0 *  Cos_inc * // remember common_factor0 = Eta_0_r * .5 
		(dPsi_r__dDldop - dPsi_b__dDldop);
}

inline double MERLIN_MilneEddington::compute_dRho_V__da()
{
	return 
		common_factor0 *  Cos_inc * // remember common_factor0 = Eta_0_r * .5 
		(dPsi_r__da - dPsi_b__da);
}

inline double MERLIN_MilneEddington::compute_dRho_V__dv()
{
	return 
		common_factor0 *  Cos_inc * // remember common_factor0 = Eta_0_r * .5 
		(dPsi_r__dv - dPsi_b__dv);
}



/////////////////////////////////////////////////////////////
//
// Derivatives of Rho_U with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dRho_U__dInclination()
{
	return 
		Cos_inc * Sin_inc * Sin_twice_azi * Eta_0_r *
		common_factor6; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_U__dAzimuth()
{
	return 
		Eta_0_r * common_factor3 * // remember Sin_inc_square * Cos_twice_azi
		common_factor6; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_U__dEta_0()
{
	return 
		.5 * common_factor4 * // remember common_factor4 = Sin_inc_square * Sin_twice_azi
		common_factor6 * dEta_0_r__dEta_0; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_U__dB()
{
	return
		common_factor8 * // remember common_factor8 = common_factor0 * common_factor4 = Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi
		(dPsi_p__dB - .5 * (dPsi_b__dB + dPsi_r__dB));
}

inline double MERLIN_MilneEddington::compute_dRho_U__dDldop()
{
	return 
		common_factor8 * // remember common_factor8 = common_factor0 * common_factor4 = Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi
		(dPsi_p__dDldop - .5 * (dPsi_b__dDldop + dPsi_r__dDldop));
}

inline double MERLIN_MilneEddington::compute_dRho_U__da()
{
	return  
		common_factor8 * // remember common_factor8 = common_factor0 * common_factor4 = Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi
		(dPsi_p__da - .5 * (dPsi_b__da + dPsi_r__da));
}

inline double MERLIN_MilneEddington::compute_dRho_U__dv()
{
	return  
		common_factor8 * // remember common_factor8 = common_factor0 * common_factor4 = Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi
		(dPsi_p__dv - 0.5 * (dPsi_b__dv + dPsi_r__dv));
}


/////////////////////////////////////////////////////////////
//
// Derivatives of Rho_Q with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dRho_Q__dInclination()
{
	return 
		Cos_inc * Sin_inc * Cos_twice_azi * Eta_0_r *
		common_factor6; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_Q__dAzimuth()
{
	return 
		-1 * Eta_0_r * common_factor4 * // remember common_factor4 = Sin_inc_square * Sin_twice_azi
		common_factor6; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_Q__dEta_0()
{
	return 
		.5 * common_factor3 * // remember common_factor3 = Sin_inc_square * Cos_twice_azi
		common_factor6 * dEta_0_r__dEta_0; // remember common_factor6 = Psi_p - common_factor5 = Psi_p - (.5 * (Psi_b + Psi_r))
}

inline double MERLIN_MilneEddington::compute_dRho_Q__dB()
{
	return
		common_factor7 * // remember common_factor7 = common_factor0 * common_factor3 = Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi
		(dPsi_p__dB - .5 * (dPsi_b__dB + dPsi_r__dB));
}

inline double MERLIN_MilneEddington::compute_dRho_Q__dDldop()
{
	return
		common_factor7 * // remember common_factor7 = common_factor0 * common_factor3 = Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi
		(dPsi_p__dDldop - .5 * (dPsi_b__dDldop + dPsi_r__dDldop));
}

inline double MERLIN_MilneEddington::compute_dRho_Q__da()
{
	return
		common_factor7 * // remember common_factor7 = common_factor0 * common_factor3 = Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi
		(dPsi_p__da - .5 * (dPsi_b__da + dPsi_r__da));
}

inline double MERLIN_MilneEddington::compute_dRho_Q__dv()
{
	return
		common_factor7 *   // remember common_factor7 = Eta_0_r * .5 * Cos_twice_azi * Sin_inc_square
		(dPsi_p__dv - 0.5 * (dPsi_b__dv + dPsi_r__dv));
}

/////////////////////////////////////////////////////////////
//
// Derivatives of Eta_V with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dEta_V__dEta_0()
{
	return
		.5 * Cos_inc * 
		common_factor10 * dEta_0_r__dEta_0; // remember common_factor10 = Phi_r - Phi_b
}

inline double MERLIN_MilneEddington::compute_dEta_V__dInclination()
{
	return
		(-1) * common_factor0 * Sin_inc *  // remember common_factor0 = Eta_0_r * .5
		common_factor10; // remember common_factor10 = Phi_r - Phi_b
}

inline double MERLIN_MilneEddington::compute_dEta_V__dB()
{
	return
		common_factor0 * Cos_inc *// remember common_factor0 = Eta_0_r * .5
		(dPhi_r__dB - dPhi_b__dB);
}

inline double MERLIN_MilneEddington::compute_dEta_V__dv()
{
	return
		common_factor0 * Cos_inc * // remember common_factor0 = Eta_0_r * .5
		(dPhi_r__dv - dPhi_b__dv);
}	

inline double MERLIN_MilneEddington::compute_dEta_V__da()
{
	return
		common_factor0 * Cos_inc *// remember common_factor0 = Eta_0_r * .5
		(dPhi_r__da - dPhi_b__da);
}

inline double MERLIN_MilneEddington::compute_dEta_V__dDldop()
{
	return
		common_factor0 * Cos_inc *// remember common_factor0 = Eta_0_r * .5
		(dPhi_r__dDldop - dPhi_b__dDldop);	
}


/////////////////////////////////////////////////////////////
//
// Derivatives of Eta_U with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dEta_U__dInclination()
{
	return
		Eta_0_r * Sin_inc * Cos_inc * Sin_twice_azi *
		common_factor2;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_U__dAzimuth()
{
	return
		common_factor3 * Eta_0_r *  // remember common_factor3 = Sin_inc_square * Cos_twice_azi
		common_factor2;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_U__dEta_0()
{
	return
		0.5 * common_factor4 *  // remember common_factor4 = Sin_twice_azi * Sin_inc_square
		common_factor2 * dEta_0_r__dEta_0;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_U__dB()
{
	return
		common_factor8 *   // remember common_factor8 = (common_factor0 * common_factor4) = Eta_0_r * .5 * Sin_twice_azi * Sin_inc_square 
		(dPhi_p__dB - .5 * (dPhi_r__dB + dPhi_b__dB));
}

inline double MERLIN_MilneEddington::compute_dEta_U__da()
{
	return
		common_factor8 *   // remember common_factor8 = (common_factor0 * common_factor4) = Eta_0_r * .5 * Sin_twice_azi * Sin_inc_square  
		(dPhi_p__da - .5 * (dPhi_b__da + dPhi_r__da));
}

inline double MERLIN_MilneEddington::compute_dEta_U__dv()
{
	return
		common_factor8 *   // remember common_factor8 = (common_factor0 * common_factor4) = Eta_0_r * .5 * Sin_twice_azi * Sin_inc_square 
		(dPhi_p__dv - .5 * (dPhi_b__dv + dPhi_r__dv));
}

inline double MERLIN_MilneEddington::compute_dEta_U__dDldop()
{
	return
		common_factor8 *   // remember common_factor8 = (common_factor0 * common_factor4) = Eta_0_r * .5 * Sin_twice_azi * Sin_inc_square 
		(dPhi_p__dDldop - .5 * (dPhi_b__dDldop + dPhi_r__dDldop));
}

/////////////////////////////////////////////////////////////
//
// Derivatives of Eta_Q with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dEta_Q__dB()
{
	return
		common_factor7 * // remember common_factor7 = (common_factor0 * common_factor3) = Eta_0_r * .5 * Cos_twice_azi * Sin_inc_square
		(dPhi_p__dB - 0.5 *(dPhi_r__dB + dPhi_b__dB));
}

inline double MERLIN_MilneEddington::compute_dEta_Q__dInclination()
{
	return 
		Eta_0_r * Sin_inc * Cos_inc * Cos_twice_azi *
		common_factor2;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_Q__dAzimuth()
{
	return
		-1 * common_factor4 * Eta_0_r *  // remember common_factor4 = Sin_inc_square * Sin_twice_azi 
		common_factor2;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_Q__dEta_0()
{
	return
		0.5 * common_factor3 *  // remember common_factor3 = Sin_inc_square * Cos_twice_azi
		common_factor2 * dEta_0_r__dEta_0;   // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_Q__da()
{
	return
		common_factor7 *   // remember common_factor7 = (common_factor0 * common_factor3) = Eta_0_r * .5 * Cos_twice_azi * Sin_inc_square
		(dPhi_p__da - .5 * (dPhi_b__da + dPhi_r__da));
}

inline double MERLIN_MilneEddington::compute_dEta_Q__dv()
{
	return
		common_factor7 *   // remember common_factor7 = (common_factor0 * common_factor3) = Eta_0_r * .5 * Cos_twice_azi * Sin_inc_square
		(dPhi_p__dv - .5 * (dPhi_b__dv + dPhi_r__dv));
}

inline double MERLIN_MilneEddington::compute_dEta_Q__dDldop()
{
	return
		common_factor7 *   // remember common_factor7 = (common_factor0 * common_factor3) = Eta_0_r * .5 * Cos_twice_azi * Sin_inc_square
		(dPhi_p__dDldop - .5 * (dPhi_b__dDldop + dPhi_r__dDldop));
}


/////////////////////////////////////////////////////////////
//
// Derivatives of Eta_I with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline double MERLIN_MilneEddington::compute_dEta_I__dv()
{
	return 
		common_factor0 * // remember common_factor0 = Eta_0_r * .5
		(
		 dPhi_p__dv * Sin_inc_square +
		 .5 * (dPhi_r__dv + dPhi_b__dv) * 
		 common_factor9 // remember common_factor9 = (1 + Cos_inc_square)
		);
}	

inline double MERLIN_MilneEddington::compute_dEta_I__dEta_0()
{
	return 
		.5 * 
		(
		 Phi_p * Sin_inc_square + 
		 common_factor1 *  // remember common_factor1 = .5 * (Phi_b + Phi_r)
		 common_factor9 // remember common_factor9 = (1 + Cos_inc_square)
		)*dEta_0_r__dEta_0;  
}

inline double MERLIN_MilneEddington::compute_dEta_I__dB()
{
	return
		common_factor0 * // remember common_factor0 = Eta_0_r * .5
		(
		 dPhi_p__dB * Sin_inc_square +
		 .5 * (dPhi_r__dB + dPhi_b__dB) *
		 common_factor9 // remember common_factor9 = (1 + Cos_inc_square)
		);
}

inline double MERLIN_MilneEddington::compute_dEta_I__dInclination()
{
	return  
		Eta_0_r * Sin_inc * Cos_inc *
		common_factor2; // remember common_factor2 = Phi_p - common_factor1 = Phi_p - .5 * (Phi_b + Phi_r)
}

inline double MERLIN_MilneEddington::compute_dEta_I__da()
{
	return 
		common_factor0 * // remember common_factor0 = Eta_0_r * .5
		(
		 dPhi_p__da * Sin_inc_square +
		 .5 * (dPhi_r__da + dPhi_b__da) * 
		 common_factor9 // remember common_factor9 = (1 + Cos_inc_square)
		);
}

inline double MERLIN_MilneEddington::compute_dEta_I__dDldop()
{
	return
		common_factor0 * // remember common_factor0 = Eta_0_r * .5
		(
		 dPhi_p__dDldop * Sin_inc_square +
		 .5 * (dPhi_r__dDldop + dPhi_b__dDldop) * 
		 common_factor9 // remember common_factor9 = (1 + Cos_inc_square)
		);

}



/////////////////////////////////////////////////////////////
//
// Derivatives of delta with respect of the free parameters
//
/////////////////////////////////////////////////////////////

inline void MERLIN_MilneEddington::compute_dDelta__dB()
{
	dDelta__dB = 
		2 * Eta_I * dEta_I__dB * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__dB - 
		 Eta_Q * dEta_Q__dB -
		 Eta_U * dEta_U__dB -
		 Eta_V * dEta_V__dB +
		 Rho_Q * dRho_Q__dB +
		 Rho_U * dRho_U__dB +
		 Rho_V * dRho_V__dB 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dB +
		 Rho_Q * dEta_Q__dB +
		 Eta_U * dRho_U__dB +
		 Rho_U * dEta_U__dB +
		 Eta_V * dRho_V__dB +
		 Rho_V * dEta_V__dB
		);
}

inline void MERLIN_MilneEddington::compute_dDelta__dInclination()
{
	dDelta__dInclination =
		2 * Eta_I * dEta_I__dInclination * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__dInclination - 
		 Eta_Q * dEta_Q__dInclination -
		 Eta_U * dEta_U__dInclination -
		 Eta_V * dEta_V__dInclination +
		 Rho_Q * dRho_Q__dInclination +
		 Rho_U * dRho_U__dInclination +
		 Rho_V * dRho_V__dInclination 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dInclination +
		 Rho_Q * dEta_Q__dInclination +
		 Eta_U * dRho_U__dInclination +
		 Rho_U * dEta_U__dInclination +
		 Eta_V * dRho_V__dInclination +
		 Rho_V * dEta_V__dInclination
		);	
}

inline void MERLIN_MilneEddington::compute_dDelta__dAzimuth()
{
	dDelta__dAzimuth =
		(Eta_I_square) * 2 * 
		( 
		 Eta_Q * dEta_Q__dAzimuth -
		 Eta_U * dEta_U__dAzimuth +
		 Rho_Q * dRho_Q__dAzimuth +
		 Rho_U * dRho_U__dAzimuth
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dAzimuth +
		 Rho_Q * dEta_Q__dAzimuth +
		 Eta_U * dRho_U__dAzimuth +
		 Rho_U * dEta_U__dAzimuth 
		);
}

inline void MERLIN_MilneEddington::compute_dDelta__dEta_0()
{
	dDelta__dEta_0 =
		2 * Eta_I * dEta_I__dEta_0 * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__dEta_0 - 
		 Eta_Q * dEta_Q__dEta_0 -
		 Eta_U * dEta_U__dEta_0 -
		 Eta_V * dEta_V__dEta_0 +
		 Rho_Q * dRho_Q__dEta_0 +
		 Rho_U * dRho_U__dEta_0 +
		 Rho_V * dRho_V__dEta_0 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dEta_0 +
		 Rho_Q * dEta_Q__dEta_0 +
		 Eta_U * dRho_U__dEta_0 +
		 Rho_U * dEta_U__dEta_0 +
		 Eta_V * dRho_V__dEta_0 +
		 Rho_V * dEta_V__dEta_0
		);
}

inline void MERLIN_MilneEddington::compute_dDelta__dDldop()
{
	dDelta__dDldop =
		2 * Eta_I * dEta_I__dDldop * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__dDldop - 
		 Eta_Q * dEta_Q__dDldop -
		 Eta_U * dEta_U__dDldop -
		 Eta_V * dEta_V__dDldop +
		 Rho_Q * dRho_Q__dDldop +
		 Rho_U * dRho_U__dDldop +
		 Rho_V * dRho_V__dDldop 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dDldop +
		 Rho_Q * dEta_Q__dDldop +
		 Eta_U * dRho_U__dDldop +
		 Rho_U * dEta_U__dDldop +
		 Eta_V * dRho_V__dDldop +
		 Rho_V * dEta_V__dDldop
		);	
}

inline void MERLIN_MilneEddington::compute_dDelta__da()
{
	dDelta__da =
		2 * Eta_I * dEta_I__da * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__da - 
		 Eta_Q * dEta_Q__da -
		 Eta_U * dEta_U__da -
		 Eta_V * dEta_V__da +
		 Rho_Q * dRho_Q__da +
		 Rho_U * dRho_U__da +
		 Rho_V * dRho_V__da 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__da +
		 Rho_Q * dEta_Q__da +
		 Eta_U * dRho_U__da +
		 Rho_U * dEta_U__da +
		 Eta_V * dRho_V__da +
		 Rho_V * dEta_V__da
		);	
}

inline double MERLIN_MilneEddington::compute_dDelta__dv(int &j)
{
	return 
		2 * Eta_I * dEta_I__dv[j] * det1  
		+ (Eta_I_square) * 2 * 
		( 
		 Eta_I * dEta_I__dv[j] - 
		 Eta_Q * dEta_Q__dv[j] -
		 Eta_U * dEta_U__dv[j] -
		 Eta_V * dEta_V__dv[j] +
		 Rho_Q * dRho_Q__dv[j] +
		 Rho_U * dRho_U__dv[j] +
		 Rho_V * dRho_V__dv[j] 
		) 
		- 2 * det3 *
		(
		 Eta_Q * dRho_Q__dv[j] +
		 Rho_Q * dEta_Q__dv[j] +
		 Eta_U * dRho_U__dv[j] +
		 Rho_U * dEta_U__dv[j] +
		 Eta_V * dRho_V__dv[j] +
		 Rho_V * dEta_V__dv[j]
		);
}

/////////////////////////////////////////////////////////////
//
// Main methods.
//
/////////////////////////////////////////////////////////////


void MERLIN_MilneEddington::get_profile (MERLIN_Model &model, MERLIN_Profile &profile, vector< vector<double> > * absortion_matrix_elements)
{  
	// Local variable. Current wavelength use in the wave loop. See for loop below.
	double Lambda;

	// Local variable. Difference between the current wave and the shifted central line normalized by the doppler width 
	double Delta_lambda;

	// Local variable. Field (Doppler units) 
	double H_doppler = model.magnetic_field.value / model.doppler_width.value; 	

	// Local variable. Inclination (Radians) 
	double Inclination = model.inclination.value; // Inclination must be already in radians.

	// Local variable. Azimuth (Radians) 
	double Azimuth = model.azimuth.value; // Azimuth must be already in radians. 


	// Clear whatever memory space is in stokes and 
	// reserve space for output vectors stokes vector.
	profile.clear();
	profile.stokes.lambda.reserve(model.n_total_wave);
	profile.stokes.I.reserve(model.n_total_wave);
	profile.stokes.Q.reserve(model.n_total_wave);
	profile.stokes.U.reserve(model.n_total_wave);
	profile.stokes.V.reserve(model.n_total_wave);


	// Initialize values for trigonometric variables
	Sin_inc = sin(Inclination);
	Sin_inc_square = Sin_inc*Sin_inc;
	Cos_inc = cos(Inclination);
	Cos_inc_square = Cos_inc*Cos_inc;
	Sin_twice_azi = sin(2.*Azimuth);
	Cos_twice_azi = cos(2.*Azimuth);  

	// Loop in spectral regions  
	for (int iregion=0; iregion < model.n_regions; iregion++)
	{
		// Loop in wavelengths for the current region
		for(int iwave=0; iwave < model.regions[iregion].n_wavelengths ; iwave++)
		{  
			Eta_I=1.; // Continuum absorption
			Eta_Q=0.;
			Eta_U=0.;
			Eta_V=0.;
			Rho_Q=0.;
			Rho_U=0.;
			Rho_V=0.;

			// current wave in Angstroms
			Lambda= model.regions[iregion].first_wavelength + (iwave * model.regions[iregion].wave_step / 1.e3);

			// Loop in the spectral lines for the given model.
			for(int iline=0; iline < model.n_lines; iline++)
			{

				Eta_0_r = model.line_strength.value * model.lines[iline].relative_strength;

				// Is the line near this wavelength? If so, use it. Ignore otherwise.
				// Notice that a line is computed if it is within MINIMUM_LINE_PROXIMITY  
				// Angstroms of the current wavelength.
				if ( ( fabs( model.lines[iline].central_wav - Lambda) < MINIMUM_LINE_PROXIMITY) )
				{

					// Compute difference between the central line and the current wave 
					// (make it dimentionless by dividing by Dldop). 
					// Quantity in milli Angstroms.
					Delta_lambda = (
							Lambda - 
							( 
							 1 + ( model.lines[iline].wavelength_shift.value * 1.e3  / CSAC::CC ) 
							) * model.lines[iline].central_wav
							) * 1.e3 
						/model.doppler_width.value;

					// compute the Voigt profile for this wave
					mvoigt(model.lines[iline].comp, model.damping.value, Delta_lambda, H_doppler, Phi_p, Psi_p, Phi_r, Psi_r, Phi_b, Psi_b);


					// Initialize the common factors used many times
					// in the equations for synthesizing a profile.
					// See the file MERLIN_CommonFactors.h
					common_factor0 = COMMON_FACTOR0;
					common_factor1 = COMMON_FACTOR1;
					common_factor2 = COMMON_FACTOR2;
					common_factor3 = COMMON_FACTOR3;
					common_factor4 = COMMON_FACTOR4;
					common_factor5 = COMMON_FACTOR5;
					common_factor6 = COMMON_FACTOR6;
					common_factor7 = COMMON_FACTOR7;
					common_factor8 = COMMON_FACTOR8;
					common_factor9 = COMMON_FACTOR9;
					common_factor10 = COMMON_FACTOR10;
					common_factor11 = COMMON_FACTOR11;

					// Compute the elements of the absorption matrix 
					// For programmers to be able to read this formulas 
					// compare the comment before the actual line of code to
					// the formulas for the elements of the absortion matrix.
					// Please refer to included Mathematica files for the
					// formulas of Eta_I, Eta_Q, Eta_U, Eta_V, Rho_Q, Rho_U, Rho_V
					// and their analytic derivatives

					//Eta_I = Eta_I + Eta_0_r * .5 * (Phi_p * Sin_inc_square +  .5 * (Phi_b + Phi_r) * (1. + Cos_inc_square) );
					Eta_I= Eta_I + common_factor0 * (Phi_p * Sin_inc_square + common_factor1 * common_factor9);

					//Eta_Q = Eta_Q + Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi * (Phi_p - .5 * (Phi_b + Phi_r)) ;
					Eta_Q= Eta_Q + common_factor7 * common_factor2;

					//Eta_U = Eta_U + Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi * (Phi_p - .5 * (Phi_b + Phi_r)) ;
					Eta_U= Eta_U + common_factor8 * common_factor2;

					//Eta_V = Eta_V + Eta_0_r * .5 * (Phi_r - Phi_b) * Cos_inc;
					Eta_V= Eta_V + common_factor0 * common_factor10 * Cos_inc;

					//Rho_Q = Rho_Q + Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi * (Psi_p - .5 * (Psi_b + Psi_r));
					Rho_Q= Rho_Q + common_factor7 * common_factor6;

					//Rho_U = Rho_U + Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi * (Psi_p - .5 * (Psi_b + Psi_r));
					Rho_U= Rho_U + common_factor8 * common_factor6;

					//Rho_V= Rho_V + Eta_0_r * .5 * (Psi_r - Psi_b) * Cos_inc;
					Rho_V= Rho_V + common_factor0 * common_factor11 * Cos_inc;

				} // end if this line was considered 
			} // end for the current spectral line from the model.

			// If we are returning the absortion matrix, the add the elements
			// We do so in the order you can see below.
			if (absortion_matrix_elements)
			{
				(*absortion_matrix_elements)[0].push_back(Eta_I);
				(*absortion_matrix_elements)[1].push_back(Eta_Q);
				(*absortion_matrix_elements)[2].push_back(Eta_U);
				(*absortion_matrix_elements)[3].push_back(Eta_V);
				(*absortion_matrix_elements)[4].push_back(Rho_Q);
				(*absortion_matrix_elements)[5].push_back(Rho_U);
				(*absortion_matrix_elements)[6].push_back(Rho_V);
			}

			// Compute other common square quantities
			Eta_I_square = Eta_I * Eta_I;
			Eta_Q_square = Eta_Q * Eta_Q;
			Eta_U_square = Eta_U * Eta_U;
			Eta_V_square = Eta_V * Eta_V;
			Rho_Q_square = Rho_Q * Rho_Q;
			Rho_U_square = Rho_U * Rho_U;
			Rho_V_square = Rho_V * Rho_V;
			common_factor12 = COMMON_FACTOR12;

			// ----------- Compute the emergent profile ------------

			// Remember, 
			// DET1 = (Eta_I_square - Eta_Q_square - Eta_U_square - Eta_V_square + Rho_Q_square + Rho_U_square + Rho_V_square)
			// is defined in MERLIN_CommonFactors.h
			// The value det1 is only used in computing Delta and the derivatives of Delta w.r.t. the free parameters, 
			// but since herewe are only computing the profile, forget about the derivatives.
			det1 = DET1;

			// Remember, 
			// DET3 = ((Eta_Q*Rho_Q)+(Eta_U*Rho_U)+(Eta_V*Rho_V)) 
			// is defined in MERLIN_CommonFactors.h
			det3 = DET3;

			// Finally we proceed to compute Delta and I, Q, U and V
			Delta = Eta_I_square * det1 - (det3 * det3);
			// Another common quantity.
			B1_divided_by_Delta = model.planck_function_gradient.value/Delta;

			// Stokes I
			I= model.planck_function.value + B1_divided_by_Delta * (Eta_I * (common_factor12));
			// Stokes Q
			Q=-B1_divided_by_Delta * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3);
			// Stokes U
			U=-B1_divided_by_Delta * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3);
			// Stokes V
			V=-B1_divided_by_Delta * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3);

			// Add this data to the profile object.
			profile.stokes.lambda.push_back(Lambda);
			profile.stokes.I.push_back(I);
			profile.stokes.Q.push_back(Q);
			profile.stokes.U.push_back(U);
			profile.stokes.V.push_back(V);
		} // End Loop in wavelengths 
	} // End Loop  in spectral regions 

	// Convolve with macroturbulence 
	// Notice that we only convolve the profile
	// thus the last parameter in the call is false.
	CSAC::MERLIN_ConvolveProfile(model, profile, false);

	// Add stray light 
	// First we get sure that the model object
	// does indeed contains a Stray Light Vector.
	if ( model.stray_light_profile.set() )
	{
		// Some debugging message.
		// cout<<"This run has stray light"<<endl;

		// We ask the stray light object
		// which is a data member of the 
		// model object to shift by the current 
		// shift value in the model.
		// This shift refers to the doppler shift
		// for the stray light.
		model.stray_light_profile.shift_light(model.stray_light_shift.value);
		// These are two local variables in this scope
		// to hold the stray light constrained to
		// match the size of the stokes vectors
		vector<double> my_stray_light_profile;
		vector<double> my_stray_light_profile_wavelength;
		// get the constrained stray light.
		model.stray_light_profile.load_constrained_stray_light_vector(my_stray_light_profile);
		// get the wavelengths for the stray light vector.
		model.stray_light_profile.load_constrained_stray_light_vector_wavelength(my_stray_light_profile_wavelength);

		// get sure that all size for the involved vectors match
		if (  
				// do the size of the stray light wavelenght vector match the size of the profile wavelength vector?
				(my_stray_light_profile_wavelength.size() == profile.stokes.lambda.size()) 
				&& 
				// do the size of the profile match the size of the constained stray light?
				(my_stray_light_profile.size() == profile.stokes.I.size()) 
		   )
		{
			int my_size = profile.stokes.I.size();
			for(int h=0; h < my_size; h++)
			{
				// Get sure that the range of the stray light vector
				// was set correctly to match the Regions defined in the model.
				if ( my_stray_light_profile_wavelength[h] != profile.stokes.lambda[h] )
				{
					char error_line[32];
					sprintf(error_line, "%i", __LINE__);
					string t_error=__FILE__ + string(":") + string(error_line) + ": the provided stray light profile does not match in wavelength";
					throw t_error;
				}
				else // Currently the Stray light only affects I,
					// later we expect to have Polarized Scatter
					// Light
					{
						if (CSAC::stray_light_model == fill) {
							// Fill Factor Formula
							profile.stokes.I[h] = model.stray_light.value * profile.stokes.I[h] + (1.0 - model.stray_light.value) * my_stray_light_profile[h];
							profile.stokes.Q[h] = model.stray_light.value * profile.stokes.Q[h];
							profile.stokes.U[h] = model.stray_light.value * profile.stokes.U[h];
							profile.stokes.V[h] = model.stray_light.value * profile.stokes.V[h];
						}
						else if (CSAC::stray_light_model == fraction)
							// Stray Light Fraction formula.
							profile.stokes.I[h] = profile.stokes.I[h] + model.stray_light.value * my_stray_light_profile[h];
						else
						{
							char error_line[32];
							sprintf(error_line, "%i", __LINE__);
							string t_error=__FILE__ + string(":") + string(error_line) + ": the stray_light_model is invalid\n";
							throw t_error;
						}
					}
			}
		}
		else  // Wrong sizes, throw and exception.
		{
			char error_line[32];
			sprintf(error_line, "%i", __LINE__);
			string t_error=__FILE__ + string(":") + string(error_line) + ": the provided constrained stray light profile does not match in size\n";
			char _tmp[32];
			sprintf(_tmp, "%i", my_stray_light_profile.size() );
			t_error += string("The size of the constrained stray light vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i", profile.stokes.I.size() );
			t_error += string("The size of the synthetic vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i", my_stray_light_profile_wavelength.size() );
			t_error += string("The size of the constrained stray light wavelength vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i",  profile.stokes.lambda.size() );
			t_error += string("The size of the synthetic  wavelength vector is: ") + _tmp + string("\n");
			throw t_error;

		}
	}


	// smear the instrument profile

}

void MERLIN_MilneEddington::get_profile_and_derivatives (MERLIN_Model &model, MERLIN_Profile &profile)
{  

	// set total number of free parameters
	model.set_n_total_free_parameters();

	// Local variable. Current wavelength use in the wave loop. See for loop below.
	double Lambda;

	// Local variable. Difference between the current wave and the shifted central line normalized by the doppler width 
	double Delta_lambda;

	// Local variable. Field (Doppler units) 
	double H_doppler = model.magnetic_field.value / model.doppler_width.value; 	

	// Local variable. Inclination (Radians) 
	double Inclination = model.inclination.value; // Inclination must be already in radians.

	// Local variable. Azimuth (Radians) 
	double Azimuth = model.azimuth.value; // Azimuth must be already in radians.  

	// Derivative of the equation H_doppler wrt magnetic field
	double dH_doppler__dB =  1 / model.doppler_width.value;

	//  Derivative of the equation H_doppler wrt doppler width
	double dH_doppler__dDldop = (-1)*(model.magnetic_field.value/(model.doppler_width.value*model.doppler_width.value));

	// We get sure that the size of the profile object alredy matches the size of the problem
	// If not we clear the given object and reset its size.
	if ( !profile.match_size(model.n_total_free_parameters, model.n_total_wave) )
	{
		profile.clear();
		profile.resize(model.n_total_free_parameters, model.n_total_wave);
	}

	// ---------------------------------------------------------------------------------------
	//
	// Study which parameters are free and proceed to insert the proper data in the derivatives order.
	int my_order = 0;
	if (model.planck_function.free) profile.stokes_derivatives.order[my_order++] = string("B0");
	if (model.planck_function_gradient.free) profile.stokes_derivatives.order[my_order++] = string("B1");
	if (model.magnetic_field.free) profile.stokes_derivatives.order[my_order++] = string("B");
	if (model.line_strength.free) profile.stokes_derivatives.order[my_order++] = string("Eta_0");
	if (model.inclination.free) profile.stokes_derivatives.order[my_order++] = string("Inclination");
	if (model.azimuth.free) profile.stokes_derivatives.order[my_order++] = string("Azimuth");
	if (model.doppler_width.free) profile.stokes_derivatives.order[my_order++] = string("Dldop");
	if (model.damping.free) profile.stokes_derivatives.order[my_order++] = string("a");
	char buffer[32];
	int h=0;
	// This loop generates names such as v1, v2, ...,vn
	// and ask if the shift in lines in the model
	// is free or not. If the shift if free for line
	// 'i' then a string v'i' is added to the derivatives
	// order in the current profile.
	for(int iline=0; iline < model.n_lines; iline++)
	{
		if (model.lines[iline].wavelength_shift.free)
		{
			sprintf(buffer, "%i", h); 
			string line_name = string("v") + buffer;
			profile.stokes_derivatives.order[my_order++] = line_name;
			h++;
		}
	}
	if (model.vmac.free) profile.stokes_derivatives.order[my_order++] = string("vmac");
	if (model.stray_light.free) profile.stokes_derivatives.order[my_order++] = string("stray_light");
	if (model.stray_light_shift.free) profile.stokes_derivatives.order[my_order++] = string("stray_light_shift");
	// End of the section where we add the strings defining the proper order.
	// ---------------------------------------------------------------------------------------


	// Initialize values for trigonometric variables
	Sin_inc = sin(Inclination);
	Sin_inc_square = Sin_inc*Sin_inc;
	Cos_inc = cos(Inclination);
	Cos_inc_square = Cos_inc*Cos_inc;
	Sin_twice_azi = sin(2.*Azimuth);
	Cos_twice_azi = cos(2.*Azimuth);  

	// This index is used to navigate along the dimension of free parameters 
	// in the vector of vectors of derivatives
	int index_free_parameter=0;
	// This index is used to navigate along the dimension total number of wavelengths 
	// in the vector of vectors of derivatives
	int index_wavelength = 0;

	// Loop in spectral regions  
	for (int iregion=0; iregion < model.n_regions; iregion++)
	{
		// Loop in wavelengths 
		for(int iwave=0; iwave < model.regions[iregion].n_wavelengths ; iwave++)
		{  
			Eta_I=1.; // Continuum absorption
			Eta_Q=0.;
			Eta_U=0.;
			Eta_V=0.;
			Rho_Q=0.;
			Rho_U=0.;
			Rho_V=0.;

			dEta_I__dv.clear();
			dEta_I__dEta_0 = 0;
			dEta_I__dB = 0;
			dEta_I__dInclination = 0;
			dEta_I__da = 0;
			dEta_I__dDldop = 0;

			dEta_Q__dv.clear();
			dEta_Q__dEta_0 = 0;
			dEta_Q__dB = 0;
			dEta_Q__dInclination = 0;
			dEta_Q__da = 0;
			dEta_Q__dDldop = 0;
			dEta_Q__dAzimuth =0;

			dEta_U__dv.clear();
			dEta_U__dEta_0 = 0;
			dEta_U__dB = 0;
			dEta_U__dInclination = 0;
			dEta_U__da = 0;
			dEta_U__dDldop = 0;
			dEta_U__dAzimuth =0;

			dEta_V__dv.clear();
			dEta_V__dEta_0 = 0;
			dEta_V__dB = 0;
			dEta_V__dInclination = 0;
			dEta_V__da = 0;
			dEta_V__dDldop = 0;

			dRho_Q__dv.clear();
			dRho_Q__dEta_0 = 0;
			dRho_Q__dB = 0;
			dRho_Q__dInclination = 0;
			dRho_Q__da = 0;
			dRho_Q__dDldop = 0;
			dRho_Q__dAzimuth =0;

			dRho_U__dv.clear();
			dRho_U__dEta_0 = 0;
			dRho_U__dB = 0;
			dRho_U__dInclination = 0;
			dRho_U__da = 0;
			dRho_U__dDldop = 0;
			dRho_U__dAzimuth =0;

			dRho_V__dv.clear();
			dRho_V__dEta_0 = 0;
			dRho_V__dB = 0;
			dRho_V__dInclination = 0;
			dRho_V__da = 0;
			dRho_V__dDldop = 0;

			dDelta__dv.clear();


			// current wave in Angstroms
			Lambda= model.regions[iregion].first_wavelength + (iwave * model.regions[iregion].wave_step/1.e3);

			// Loop in spectral lines 
			for(int iline=0; iline < model.n_lines; iline++)
			{

				Eta_0_r = model.line_strength.value * model.lines[iline].relative_strength;
				dEta_0_r__dEta_0=model.lines[iline].relative_strength;

				// Is the line near this wavelength? 
				if ( ( fabs( model.lines[iline].central_wav - Lambda) < MINIMUM_LINE_PROXIMITY) )
				{

					// Compute difference between the central line and the current wave 
					// (make it dimentionless by dividing by Dldop). 
					// Quantity in milli Angstroms.
					Delta_lambda = (
							Lambda - 
							( 
							 1 + ( model.lines[iline].wavelength_shift.value * 1.e3  / CSAC::CC ) 
							) * model.lines[iline].central_wav
							) * 1.e3 
						/model.doppler_width.value;

					// Obtain the derivative of delta_lambda wrt doppler width
					double dDelta_lambda__dDldop = (-1) * (Delta_lambda /model.doppler_width.value);
					// Obtain the derivative of delta_lambda wrt central wave length shift
					double dDelta_lambda__dv = (-1 * 1.e6 *  model.lines[iline].central_wav) / (CSAC::CC * model.doppler_width.value);

					// compute the Voigt profile for this wave
					mvoigt_d ( 
							model.lines[iline].comp, 
							model.damping.value, 
							Delta_lambda, 
							H_doppler, 
							Phi_p, 
							Psi_p, 
							Phi_r, 
							Psi_r, 
							Phi_b, 
							Psi_b,
							dH_doppler__dB,
							dH_doppler__dDldop,
							dDelta_lambda__dDldop,
							dDelta_lambda__dv,
							dPhi_p__dv,
							dPhi_r__dv,
							dPhi_b__dv,
							dPhi_p__dB,
							dPhi_r__dB,
							dPhi_b__dB,
							dPhi_p__da,
							dPhi_r__da,
							dPhi_b__da,
							dPhi_p__dDldop,
							dPhi_r__dDldop,
							dPhi_b__dDldop,
							dPsi_p__dB,
							dPsi_b__dB,
							dPsi_r__dB,
							dPsi_p__dv,
							dPsi_r__dv,
							dPsi_b__dv,
							dPsi_p__da,
							dPsi_b__da,
							dPsi_r__da,
							dPsi_p__dDldop,
							dPsi_b__dDldop,
							dPsi_r__dDldop
								);			

					common_factor0 = COMMON_FACTOR0;
					common_factor1 = COMMON_FACTOR1;
					common_factor2 = COMMON_FACTOR2;
					common_factor3 = COMMON_FACTOR3;
					common_factor4 = COMMON_FACTOR4;
					common_factor5 = COMMON_FACTOR5;
					common_factor6 = COMMON_FACTOR6;
					common_factor7 = COMMON_FACTOR7;
					common_factor8 = COMMON_FACTOR8;
					common_factor9 = COMMON_FACTOR9;
					common_factor10 = COMMON_FACTOR10;
					common_factor11 = COMMON_FACTOR11;

					// Compute the elements of the absorption matrix 
					// For programmers to be able to read this formulas 
					// compare the comment before the actual line of code to
					// the formulas for the elements of the absortion matrix.
					// Please refer to included Mathematica files for the
					// formulas of Eta_I, Eta_Q, Eta_U, Eta_V, Rho_Q, Rho_U, Rho_V
					// and their analytic derivatives

					//Eta_I = Eta_I + Eta_0_r * .5 * (Phi_p * Sin_inc_square +  .5 * (Phi_b + Phi_r) * (1. + Cos_inc_square) );
					Eta_I= Eta_I + common_factor0 * (Phi_p * Sin_inc_square + common_factor1 * common_factor9);

					//Eta_Q = Eta_Q + Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi * (Phi_p - .5 * (Phi_b + Phi_r)) ;
					Eta_Q= Eta_Q + common_factor7 * common_factor2;

					//Eta_U = Eta_U + Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi * (Phi_p - .5 * (Phi_b + Phi_r)) ;
					Eta_U= Eta_U + common_factor8 * common_factor2;

					//Eta_V = Eta_V + Eta_0_r * .5 * (Phi_r - Phi_b) * Cos_inc;
					Eta_V= Eta_V + common_factor0 * common_factor10 * Cos_inc;

					//Rho_Q = Rho_Q + Eta_0_r * .5 * Sin_inc_square * Cos_twice_azi * (Psi_p - .5 * (Psi_b + Psi_r));
					Rho_Q= Rho_Q + common_factor7 * common_factor6;

					//Rho_U = Rho_U + Eta_0_r * .5 * Sin_inc_square * Sin_twice_azi * (Psi_p - .5 * (Psi_b + Psi_r));
					Rho_U= Rho_U + common_factor8 * common_factor6;

					//Rho_V= Rho_V + Eta_0_r * .5 * (Psi_r - Psi_b) * Cos_inc;
					Rho_V= Rho_V + common_factor0 * common_factor11 * Cos_inc;

					// Compute derivatives with respect to the line strength
					if (model.line_strength.free)
					{
						dEta_I__dEta_0 = dEta_I__dEta_0 + compute_dEta_I__dEta_0();
						dEta_Q__dEta_0 = dEta_Q__dEta_0 + compute_dEta_Q__dEta_0();
						dEta_U__dEta_0 = dEta_U__dEta_0 + compute_dEta_U__dEta_0();
						dEta_V__dEta_0 = dEta_V__dEta_0 + compute_dEta_V__dEta_0();
						dRho_Q__dEta_0 = dRho_Q__dEta_0 + compute_dRho_Q__dEta_0();
						dRho_U__dEta_0 = dRho_U__dEta_0 + compute_dRho_U__dEta_0();
						dRho_V__dEta_0 = dRho_V__dEta_0 + compute_dRho_V__dEta_0();
					}

					// Compute derivative with respect to the field
					if (model.magnetic_field.free)
					{
						dEta_I__dB = dEta_I__dB + compute_dEta_I__dB();
						dEta_Q__dB = dEta_Q__dB + compute_dEta_Q__dB();
						dEta_U__dB = dEta_U__dB + compute_dEta_U__dB();
						dEta_V__dB = dEta_V__dB + compute_dEta_V__dB();
						dRho_Q__dB = dRho_Q__dB + compute_dRho_Q__dB();
						dRho_U__dB = dRho_U__dB + compute_dRho_U__dB();
						dRho_V__dB = dRho_V__dB + compute_dRho_V__dB();
					}

					// Compute derivatives with respect to the inclination
					if (model.inclination.free)
					{
						dEta_I__dInclination = dEta_I__dInclination + compute_dEta_I__dInclination();
						dEta_Q__dInclination = dEta_Q__dInclination + compute_dEta_Q__dInclination();
						dEta_U__dInclination = dEta_U__dInclination + compute_dEta_U__dInclination();
						dEta_V__dInclination = dEta_V__dInclination + compute_dEta_V__dInclination();
						dRho_Q__dInclination = dRho_Q__dInclination + compute_dRho_Q__dInclination();
						dRho_U__dInclination = dRho_U__dInclination + compute_dRho_U__dInclination();
						dRho_V__dInclination = dRho_V__dInclination + compute_dRho_V__dInclination();
					}

					// Compute derivatives with respect to the azimuth
					if (model.azimuth.free)
					{					
						dEta_Q__dAzimuth = dEta_Q__dAzimuth + compute_dEta_Q__dAzimuth();
						dEta_U__dAzimuth = dEta_U__dAzimuth + compute_dEta_U__dAzimuth();
						dRho_Q__dAzimuth = dRho_Q__dAzimuth + compute_dRho_Q__dAzimuth();
						dRho_U__dAzimuth = dRho_U__dAzimuth + compute_dRho_U__dAzimuth();
					}

					// Compute derivatives with respect to the damping
					if (model.damping.free)
					{	
						dEta_I__da = dEta_I__da + compute_dEta_I__da();
						dEta_Q__da = dEta_Q__da + compute_dEta_Q__da();
						dEta_U__da = dEta_U__da + compute_dEta_U__da();
						dEta_V__da = dEta_V__da + compute_dEta_V__da();
						dRho_Q__da = dRho_Q__da + compute_dRho_Q__da();
						dRho_U__da = dRho_U__da + compute_dRho_U__da();
						dRho_V__da = dRho_V__da + compute_dRho_V__da();
					}

					// Compute derivatives with respect to the doppler width
					if (model.doppler_width.free)
					{	
						dEta_I__dDldop = dEta_I__dDldop + compute_dEta_I__dDldop();
						dEta_Q__dDldop = dEta_Q__dDldop + compute_dEta_Q__dDldop();
						dEta_U__dDldop = dEta_U__dDldop + compute_dEta_U__dDldop();
						dEta_V__dDldop = dEta_V__dDldop + compute_dEta_V__dDldop();
						dRho_Q__dDldop = dRho_Q__dDldop + compute_dRho_Q__dDldop();
						dRho_U__dDldop = dRho_U__dDldop + compute_dRho_U__dDldop();
						dRho_V__dDldop = dRho_V__dDldop + compute_dRho_V__dDldop();
					}

					// Compute derivatives with respect to the wavelength shift for the current spectral line
					if (model.lines[iline].wavelength_shift.free)
					{
						dEta_I__dv.push_back(compute_dEta_I__dv());
						dEta_Q__dv.push_back(compute_dEta_Q__dv());
						dEta_U__dv.push_back(compute_dEta_U__dv());
						dEta_V__dv.push_back(compute_dEta_V__dv());
						dRho_Q__dv.push_back(compute_dRho_Q__dv());
						dRho_U__dv.push_back(compute_dRho_U__dv());
						dRho_V__dv.push_back(compute_dRho_V__dv());
					}

				} // end if this line was considered 

			} // end for in spectral lines  

			// Compute some common square quantities
			Eta_I_square = Eta_I * Eta_I;
			Eta_Q_square = Eta_Q * Eta_Q;
			Eta_U_square = Eta_U * Eta_U;
			Eta_V_square = Eta_V * Eta_V;
			Rho_Q_square = Rho_Q * Rho_Q;
			Rho_U_square = Rho_U * Rho_U;
			Rho_V_square = Rho_V * Rho_V;
			common_factor12 = COMMON_FACTOR12;

			// remember, det1 = (Eta_I_square - Eta_Q_square - Eta_U_square - Eta_V_square + Rho_Q_square + Rho_U_square + Rho_V_square)
			// det1 is only used in computing Delta and the derivatives of Delta w.r.t. the free parameters.
			det1=DET1;
			// remember, det3 = ((Eta_Q*Rho_Q)+(Eta_U*Rho_U)+(Eta_V*Rho_V)) 
			det3=DET3;

			Delta = Eta_I_square * det1 - (det3 * det3);
			Delta_square = Delta * Delta;	

			// Compute derivative of delta with respect to the field
			if (model.magnetic_field.free) compute_dDelta__dB();
			// Compute derivative of delta with respect to the inclination angle
			if (model.inclination.free) compute_dDelta__dInclination();
			// Compute derivative of delta with respect to the azimuth angle
			if (model.azimuth.free) compute_dDelta__dAzimuth();	
			// Compute derivative of delta with respect to the line strength
			if (model.line_strength.free) compute_dDelta__dEta_0();
			// Compute derivative of delta with respect to the doppler width
			if (model.doppler_width.free) compute_dDelta__dDldop();
			// Compute derivative of delta with respect to the damping
			if (model.damping.free) compute_dDelta__da();
			// Compute derivative of delta with respect to the wavelength shift of each spectral line
			// Notice that only the lines that are free are included in the output
			// since within the loop for spectral lines, we only added derivatives of the
			// elements of the absortion matrix when the line was a free parameter
			h=0;
			for(int iline=0; iline < model.n_lines; iline++)
				if (model.lines[iline].wavelength_shift.free)
				{
					dDelta__dv.push_back(compute_dDelta__dv(h)); 
					h++;
				}


			// This quantity is used locally when computing I, Q, U and V.
			B1_divided_by_Delta = model.planck_function_gradient.value / Delta;
			// This quantity is used in the derivative formulas
			B1_divided_by_Delta_square = model.planck_function_gradient.value / Delta_square;

			// ------------------ Vector I and its derivatives -------------------------------------------
			//
			// I = model.planck_function.value + B1_divided_by_Delta * (Eta_I * (Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square));
			// or
			// I = model.planck_function.value + B1_divided_by_Delta * (Eta_I * (common_factor12));
			// Add value of I for this wave and its derivatives for this wave
			profile.stokes.I[index_wavelength] = ( model.planck_function.value + B1_divided_by_Delta * (Eta_I * (common_factor12)) );
			// since we are computing derivatives for all free parameters, reset the index for the free parameters
			index_free_parameter = 0;
			// compute derivative of I w.r.t planck function, notice that this derivatives is constant 1.0
			if (model.planck_function.free)	profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = (double) 2/2;
			// compute derivative of I w.r.t planck function gradient
			if (model.planck_function_gradient.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dB1();
			// compute derivative of I w.r.t magnetic field
			if (model.magnetic_field.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dB();
			// compute derivative of I w.r.t line strength
			if (model.line_strength.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dEta_0();
			// compute derivative of I w.r.t inclination
			if (model.inclination.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dInclination();
			// compute derivative of I w.r.t azimuth
			if (model.azimuth.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dAzimuth();
			// compute derivative of I w.r.t doppler width
			if (model.doppler_width.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dDldop();
			// compute derivative of I w.r.t damping
			if (model.damping.free) profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__da();
			// compute derivative of I w.r.t spectral line wave shift
			h=0;
			for(int iline=0; iline < model.n_lines; iline++)
			{
				if (model.lines[iline].wavelength_shift.free)
				{
					profile.stokes_derivatives.I_dxdy[index_free_parameter++][index_wavelength] = compute_dI__dv(h); 
					h++;
				}
			}
			// ----------------- End Vector I and its derivatives ----------------------------------------

			// ------------------ Vector Q and its derivatives -------------------------------------------
			//
			// Q = -B1_divided_by_Delta * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3);
			profile.stokes.Q[index_wavelength] = ( -B1_divided_by_Delta * (Eta_I_square * Eta_Q + Eta_I * (Eta_V * Rho_U - Eta_U * Rho_V) + Rho_Q * det3) );
			// since we are computing derivatives for all free parameters, reset the index for the free parameters
			index_free_parameter = 0;
			// compute derivative of Q w.r.t planck function, notice that this derivatives is constant 0
			if (model.planck_function.free)	profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = (double) (1 * 0);
			// compute derivative of Q w.r.t planck function gradient
			if (model.planck_function_gradient.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dB1();
			// compute derivative of Q w.r.t magnetic field
			if (model.magnetic_field.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dB();
			// compute derivative of Q w.r.t line strength
			if (model.line_strength.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dEta_0();
			// compute derivative of Q w.r.t inclination
			if (model.inclination.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dInclination();
			// compute derivative of Q w.r.t azimuth
			if (model.azimuth.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dAzimuth();
			// compute derivative of Q w.r.t doppler width
			if (model.doppler_width.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dDldop();
			// compute derivative of Q w.r.t damping
			if (model.damping.free) profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__da();
			// compute derivative of Q w.r.t spectral line wave shift
			h=0;
			for(int iline=0; iline < model.n_lines; iline++)
			{
				if (model.lines[iline].wavelength_shift.free)
				{
					profile.stokes_derivatives.Q_dxdy[index_free_parameter++][index_wavelength] = compute_dQ__dv(h); 
					h++;
				}
			}
			// ----------------- End Vector Q and its derivatives ----------------------------------------

			// ------------------ Vector U and its derivatives -------------------------------------------
			//
			// U = -B1_divided_by_Delta * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3);
			profile.stokes.U[index_wavelength] = ( -B1_divided_by_Delta * (Eta_I_square * Eta_U + Eta_I * (Eta_Q * Rho_V - Eta_V * Rho_Q) + Rho_U * det3) );
			// since we are computing derivatives for all free parameters, reset the index for the free parameters
			index_free_parameter = 0;
			// compute derivative of U w.r.t planck function, notice that this derivatives is constant 0
			if (model.planck_function.free)	profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = (double) (1 * 0);
			// compute derivative of U w.r.t planck function gradient
			if (model.planck_function_gradient.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dB1();
			// compute derivative of U w.r.t magnetic field
			if (model.magnetic_field.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dB();
			// compute derivative of U w.r.t line strength
			if (model.line_strength.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dEta_0();
			// compute derivative of U w.r.t inclination
			if (model.inclination.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dInclination();
			// compute derivative of U w.r.t azimuth
			if (model.azimuth.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dAzimuth();
			// compute derivative of U w.r.t doppler width
			if (model.doppler_width.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dDldop();
			// compute derivative of U w.r.t damping
			if (model.damping.free) profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__da();
			// compute derivative of U w.r.t spectral line wave shift
			h=0;
			for(int iline=0; iline < model.n_lines; iline++)
			{
				if (model.lines[iline].wavelength_shift.free)
				{
					profile.stokes_derivatives.U_dxdy[index_free_parameter++][index_wavelength] = compute_dU__dv(h); 
					h++;
				}
			}
			// ----------------- End Vector Q and its derivatives ----------------------------------------

			// ------------------ Vector V and its derivatives -------------------------------------------
			//
			// V = -B1_divided_by_Delta * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3);
			profile.stokes.V[index_wavelength] = ( -B1_divided_by_Delta * (Eta_I_square * Eta_V + Eta_I * (Eta_U * Rho_Q - Eta_Q * Rho_U) + Rho_V * det3) );
			// since we are computing derivatives for all free parameters, reset the index for the free parameters
			index_free_parameter = 0;
			// compute derivative of V w.r.t planck function, notice that this derivatives is constant 0
			if (model.planck_function.free)	profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = (double) (1 * 0);
			// compute derivative of V w.r.t planck function gradient
			if (model.planck_function_gradient.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dB1();
			// compute derivative of V w.r.t magnetic field
			if (model.magnetic_field.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dB();
			// compute derivative of V w.r.t line strength
			if (model.line_strength.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dEta_0();
			// compute derivative of V w.r.t inclination
			if (model.inclination.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dInclination();
			// compute derivative of V w.r.t azimuth
			if (model.azimuth.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dAzimuth();
			// compute derivative of V w.r.t doppler width
			if (model.doppler_width.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dDldop();
			// compute derivative of V w.r.t damping
			if (model.damping.free) profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__da();
			// compute derivative of V w.r.t spectral line wave shift
			h=0;
			for(int iline=0; iline < model.n_lines; iline++)
			{
				if (model.lines[iline].wavelength_shift.free)
				{
					profile.stokes_derivatives.V_dxdy[index_free_parameter++][index_wavelength] = compute_dV__dv(h); 
					h++;
				}
			}
			// ----------------- End Vector V and its derivatives ----------------------------------------

			// Add the wavelength for which I, U, U and V along its derivatives was computed to corresponding vector
			profile.stokes.lambda[index_wavelength] = Lambda;
			// increase the index for the wavelength
			index_wavelength++;

		} // End Loop in wavelengths 

	} // End Loop  in spectral regions 

	// Convolve with macroturbulence 
	CSAC::MERLIN_ConvolveProfile(model, profile, true);
	// Notice, the index for the free parameters is set to the macroturbulence
	// since it was last modified by the section of code adding V and its derivatives.
	// If the macroturbulence is a free parameter, the the call to convolve profile 
	// added the derivative, thus we need to move the index forward.
	if (model.vmac.free) index_free_parameter++;

	// Add the stray light. 
	if ( model.stray_light_profile.set() )
	{
		// Some debugging message.
		// cout<<"This run has stray light"<<endl;

		// We ask the stray light object
		// which is a data member of the 
		// model object to shift by the current 
		// shift value in the model.
		// This shift refers to the doppler shift
		// for the stray light.
		model.stray_light_profile.shift_light(model.stray_light_shift.value);
		// These are two local variables in this scope
		// to hold the stray light constrained to
		// match the size of the stokes vectors
		vector<double> my_stray_light_profile;
		vector<double> my_stray_light_profile_wavelength;
		// get the constrained stray light.
		model.stray_light_profile.load_constrained_stray_light_vector(my_stray_light_profile);
		// get the wavelengths for the stray light vector.
		model.stray_light_profile.load_constrained_stray_light_vector_wavelength(my_stray_light_profile_wavelength);

		// get sure that all size for the involved vectors match
		if (  
				// do the size of the stray light wavelenght vector match the size of the profile wavelength vector?
				(my_stray_light_profile_wavelength.size() == profile.stokes.lambda.size()) 
				&& 
				// do the size of the profile match the size of the constained stray light?
				(my_stray_light_profile.size() == profile.stokes.I.size()) 
		   )
		{
			// save the synthesized profiles for use in the derivative calculation
			MERLIN_Profile synth_profile;
			synth_profile.stokes.I = profile.stokes.I;
			synth_profile.stokes.Q = profile.stokes.Q;
			synth_profile.stokes.U = profile.stokes.U;
			synth_profile.stokes.V = profile.stokes.V;

			int my_size = profile.stokes.I.size();

			for(int hh = 0; hh < my_size; hh++)
			{
				if ( my_stray_light_profile_wavelength[hh] != profile.stokes.lambda[hh] )
				{
					char error_line[32];
					sprintf(error_line, "%i", __LINE__);
					string t_error=__FILE__ + string(":") + string(error_line) + ": the provided stray light profile does not match in wavelength";
					throw t_error;
				}
				else
				{
					// Currently the Stray light only affects I, later
					// we expect to have Polarized Scatter Light

					if (CSAC::stray_light_model == fill) {
						// Fill Factor Formula
						profile.stokes.I[hh] = model.stray_light.value * profile.stokes.I[hh] + (1.0 - model.stray_light.value) * my_stray_light_profile[hh];
						profile.stokes.Q[hh] = model.stray_light.value * profile.stokes.Q[hh];
						profile.stokes.U[hh] = model.stray_light.value * profile.stokes.U[hh];
						profile.stokes.V[hh] = model.stray_light.value * profile.stokes.V[hh];
					}
					else if (CSAC::stray_light_model == fraction)
						// Stray Light Fraction formula.
						profile.stokes.I[hh] = profile.stokes.I[hh] + model.stray_light.value * my_stray_light_profile[hh];
					else
					{
						char error_line[32];
						sprintf(error_line, "%i", __LINE__);
						string t_error=__FILE__ + string(":") + string(error_line) + ": the stray_light_model is invalid\n";
						throw t_error;
					}
					// Now we go for the derivative for the stray light
					if (model.stray_light.free)
					{
						if (CSAC::stray_light_model == fill) {
							// Derivative Fill Factor Formula
							profile.stokes_derivatives.I_dxdy[index_free_parameter][hh] = synth_profile.stokes.I[hh] - my_stray_light_profile[hh];
							profile.stokes_derivatives.Q_dxdy[index_free_parameter][hh] = synth_profile.stokes.Q[hh];
							profile.stokes_derivatives.U_dxdy[index_free_parameter][hh] = synth_profile.stokes.U[hh];
							profile.stokes_derivatives.V_dxdy[index_free_parameter][hh] = synth_profile.stokes.U[hh];
						}
						else if (CSAC::stray_light_model == fraction)
							// Derivative Stray Light Fraction formula.
							profile.stokes_derivatives.I_dxdy[index_free_parameter][hh] = my_stray_light_profile[hh];
						else
						{
							char error_line[32];
							sprintf(error_line, "%i", __LINE__);
							string t_error=__FILE__ + string(":") + string(error_line) + ": the stray_light_model is invalid\n";
							throw t_error;
						}
						profile.stokes_derivatives.Q_dxdy[index_free_parameter][hh] = (double) (1 * 0);
						profile.stokes_derivatives.U_dxdy[index_free_parameter][hh] = (double) (1 * 0);
						profile.stokes_derivatives.V_dxdy[index_free_parameter][hh] = (double) (1 * 0);
					}
				}
			}
			// If the stray light was a free parameter we must increase the corresponding index.
			if (model.stray_light.free) index_free_parameter++;

			// Now compute if necessary the derivative with respect to the shift of the stray light
			if(model.stray_light_shift.free)  
			{
				vector <double> my_derivative;
				model.stray_light_profile.load_constrained_derivative_wrt_current_shift(my_derivative);
				for (int hh = 0; hh < my_size; hh++)
				{
					profile.stokes_derivatives.I_dxdy[index_free_parameter][hh] = (1.0 - model.stray_light.value) * my_derivative[hh];
					profile.stokes_derivatives.Q_dxdy[index_free_parameter][hh] = (double) (1 * 0);
					profile.stokes_derivatives.U_dxdy[index_free_parameter][hh] = (double) (1 * 0);
					profile.stokes_derivatives.V_dxdy[index_free_parameter][hh] = (double) (1 * 0);
				}
			}
		}	
		else
		{
			char error_line[32];
			sprintf(error_line, "%i", __LINE__);
			string t_error=__FILE__ + string(":") + string(error_line) + ": the provided constrained stray light profile does not match in size\n";
			char _tmp[32];
			sprintf(_tmp, "%i", my_stray_light_profile.size() );
			t_error += string("The size of the constrained stray light vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i", profile.stokes.I.size() );
			t_error += string("The size of the synthetic vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i", my_stray_light_profile_wavelength.size() );
			t_error += string("The size of the constrained stray light wavelength vector is: ") + _tmp + string("\n");
			sprintf(_tmp, "%i",  profile.stokes.lambda.size() );
			t_error += string("The size of the synthetic  wavelength vector is: ") + _tmp + string("\n");
			throw t_error;
		}
	}

	apply_filter();
	// smear the instrument profile
}

