/* ============================================================================
 * Language: C++ 
 * File: MERLIN_MilneEddington.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_MilneEddington_h_
#define MERLIN_MilneEddington_h_ 1

#include <vector>
using std::vector;

#include <iostream>
using std::endl;
using std::cout;

#include "MERLIN_Model.h"
using CSAC::MERLIN_Model;
#include "MERLIN_Profile.h"
using CSAC::MERLIN_Profile;




namespace CSAC
{
  
    /// Compute theoretical Milne-Eddington profiles.
    class MERLIN_MilneEddington
    {
	/// Line Strength for current line (dimentionless)		
	double Eta_0_r;	
	/// Derivative of the relative line strength for the current line w.r.t the absolute line strength
	double dEta_0_r__dEta_0;	
	
	/// denominator used to compute the Stoke profiles 
	double Delta; 
	/// Stoke profiles				
	double I, Q, U, V;				
	/// Absortion and dispersion profiles
        double Phi_r, Psi_r, Phi_b, Psi_b, Phi_p, Psi_p;
        ///Elements of the absortion matrix for the RTE equation
        double Eta_I, Eta_Q, Eta_U, Eta_V, Rho_Q, Rho_U, Rho_V; 
	
	/// Derivatives of I 
	/// compute derivative of I w.r.t planck function gradient
	double compute_dI__dB1();
	/// compute derivative of I w.r.t magnetic field
	double compute_dI__dB();
	/// compute derivative of I w.r.t line strength
	double compute_dI__dEta_0();
	/// compute derivative of I w.r.t doppler width
	double compute_dI__dDldop();
	/// compute derivative of I w.r.t inclination
	double compute_dI__dInclination();
	/// compute derivative of I w.r.t inclination
	double compute_dI__dAzimuth();
	/// compute derivative of I w.r.t damping
	double compute_dI__da();
	/// compute derivative of I w.r.t spectral line wave shift
	double compute_dI__dv(int &);
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	/// Derivatives of Q 
	/// compute derivative of Q w.r.t planck function gradient
	double compute_dQ__dB1();
	/// compute derivative of Q w.r.t magnetic field
	double compute_dQ__dB();
	/// compute derivative of Q w.r.t line strength
	double compute_dQ__dEta_0();
	/// compute derivative of Q w.r.t doppler width
	double compute_dQ__dDldop();
	/// compute derivative of Q w.r.t inclination
	double compute_dQ__dInclination();
	/// compute derivative of Q w.r.t inclination
	double compute_dQ__dAzimuth();
	/// compute derivative of Q w.r.t damping
	double compute_dQ__da();
	/// compute derivative of Q w.r.t spectral line wave shift
	double compute_dQ__dv(int &);
	///////////////////////////////////////////////////////////////////////////////////////////
	
	/// Derivatives of U 
	/// compute derivative of U w.r.t planck function gradient
	double compute_dU__dB1();
	/// compute derivative of U w.r.t magnetic field
	double compute_dU__dB();
	/// compute derivative of U w.r.t line strength
	double compute_dU__dEta_0();
	/// compute derivative of U w.r.t doppler width
	double compute_dU__dDldop();
	/// compute derivative of U w.r.t inclination
	double compute_dU__dInclination();
	/// compute derivative of U w.r.t inclination
	double compute_dU__dAzimuth();
	/// compute derivative of U w.r.t damping
	double compute_dU__da();
	/// compute derivative of U w.r.t spectral line wave shift
	double compute_dU__dv(int &);
	///////////////////////////////////////////////////////////////////////////////////////////
	
	/// Derivatives of V 
	/// compute derivative of V w.r.t planck function gradient
	double compute_dV__dB1();
	/// compute derivative of V w.r.t magnetic field
	double compute_dV__dB();
	/// compute derivative of V w.r.t line strength
	double compute_dV__dEta_0();
	/// compute derivative of V w.r.t doppler width
	double compute_dV__dDldop();
	/// compute derivative of V w.r.t inclination
	double compute_dV__dInclination();
	/// compute derivative of V w.r.t inclination
	double compute_dV__dAzimuth();
	/// compute derivative of V w.r.t damping
	double compute_dV__da();
	/// compute derivative of V w.r.t spectral line wave shift
	double compute_dV__dv(int &);
	///////////////////////////////////////////////////////////////////////////////////////////
  
        /// Derivatives of the denominator (delta) for I, Q, U, V 
        double dDelta__dB; 
        double dDelta__dInclination;  
        double dDelta__dAzimuth;  
        double dDelta__dEta_0;  
        double dDelta__dDldop; 
        double dDelta__da; 
        vector<double> dDelta__dv; 
        /// Compute derivative of delta with respect to the field
        void compute_dDelta__dB();
        /// Compute derivative of delta with respect to the inclination angle
        void compute_dDelta__dInclination();
        /// Compute derivative of delta with respect to the azimuth angle
        void compute_dDelta__dAzimuth();	
        /// Compute derivative of delta with respect to the line strength
        void compute_dDelta__dEta_0();
        /// Compute derivative of delta with respect to the doppler width
        void compute_dDelta__dDldop();
        /// Compute derivative of delta with respect to the damping
        void compute_dDelta__da();
        /// Compute derivative of delta with respect to the wavelength shift of each spectral line
        double compute_dDelta__dv(int &);
        ///////////////////////////////////////////////////////////////////////////////////////////
	
        /// Derivatives of Eta_I (element of the absortion matrix)    
        double dEta_I__dB; 
        double dEta_I__dInclination; 
        double dEta_I__dEta_0; 
        double dEta_I__da; 
        vector <double> dEta_I__dv; 
        double dEta_I__dDldop; 
	/// Compute derivative of Eta_I with respect to the wavelength shift
        double compute_dEta_I__dv();
        /// Compute derivative of Eta_I with respect to the line strength
        double compute_dEta_I__dEta_0();
        /// Compute derivative of Eta_I with respect to the field
        double compute_dEta_I__dB();
        /// Compute derivative of Eta_I with respect to the inclination
        double compute_dEta_I__dInclination();
        /// Compute derivative of Eta_I with respect to the damping
        double compute_dEta_I__da();
        /// Compute derivative of Eta_I with respect to the doppler width
        double compute_dEta_I__dDldop();
        ///////////////////////////////////////////////////////////////////////////////////////////
  
  
  
        /// Derivatives of Eta_Q (element of the absortion matrix) 
	double dEta_Q__dB;
	double dEta_Q__dInclination;
	double dEta_Q__dAzimuth;
	double dEta_Q__dEta_0;
	double dEta_Q__da;
	vector<double> dEta_Q__dv;
	double dEta_Q__dDldop;
	/// Compute derivative of Eta_Q with respect to the field
        double compute_dEta_Q__dB();
        /// Compute derivative of Eta_Q with respect to the inclination
        double compute_dEta_Q__dInclination();
        /// Compute derivative of Eta_Q with respect to the azimuth
        double compute_dEta_Q__dAzimuth();
        /// Compute derivative of Eta_Q with respect to the line strength
        double compute_dEta_Q__dEta_0();
	/// Compute derivative of Eta_Q with respect to the damping
        double compute_dEta_Q__da();
	/// Compute derivative of Eta_Q with respect to the wavelength shift
        double compute_dEta_Q__dv();
	/// Compute derivative of Eta_Q with respect to the doppler_width
        double compute_dEta_Q__dDldop();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	/// Derivatives of Eta_U (element of the absortion matrix) 
	double dEta_U__dB;
	double dEta_U__dInclination;
	double dEta_U__dAzimuth;
	double dEta_U__dEta_0;
	double dEta_U__dDldop;
	double dEta_U__da;
	vector<double> dEta_U__dv;
	/// Compute derivative of Eta_U with respect to the field
        double compute_dEta_U__dB();
        /// Compute derivative of Eta_U with respect to the inclination
        double compute_dEta_U__dInclination();
        /// Compute derivative of Eta_U with respect to the azimuth
        double compute_dEta_U__dAzimuth();
        /// Compute derivative of Eta_U with respect to the line strength
        double compute_dEta_U__dEta_0();
	/// Compute derivative of Eta_U with respect to the damping
        double compute_dEta_U__da();
	/// Compute derivative of Eta_U with respect to the wavelength shift
        double compute_dEta_U__dv();
	/// Compute derivative of Eta_U with respect to the doppler_width
        double compute_dEta_U__dDldop();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	/// Derivatives of Eta_V (element of the absortion matrix) 
	double dEta_V__dB;
	double dEta_V__dInclination;
	double dEta_V__dEta_0;
	double dEta_V__dDldop;
	double dEta_V__da;
	vector<double> dEta_V__dv;
        /// Compute derivative of Eta_V with respect to the wavelength shift
        double compute_dEta_V__dv();
        /// Compute derivative of Eta_V with respect to the line strength
        double compute_dEta_V__dEta_0();
        /// Compute derivative of Eta_V with respect to the field
        double compute_dEta_V__dB();
        /// Compute derivative of Eta_V with respect to the inclination
        double compute_dEta_V__dInclination();
        /// Compute derivative of Eta_V with respect to the damping
        double compute_dEta_V__da();
        /// Compute derivative of Eta_V with respect to the doppler width
        double compute_dEta_V__dDldop();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	/// Derivatives of Rho_Q (element of the absortion matrix)    
	double dRho_Q__dB;
	double dRho_Q__dInclination;
	double dRho_Q__dAzimuth;
	double dRho_Q__dEta_0;
	double dRho_Q__dDldop;
	double dRho_Q__da;
	vector<double> dRho_Q__dv;
	/// Compute derivative of Rho_Q with respect to the field
	double compute_dRho_Q__dB();
	/// Compute derivative of Rho_Q with respect to the inclination
	double compute_dRho_Q__dInclination();
	/// Compute derivative of Rho_Q with respect to the azimuth
	double compute_dRho_Q__dAzimuth();
	/// Compute derivative of Rho_Q with respect to the line strength
	double compute_dRho_Q__dEta_0();
	/// Compute derivative of Rho_Q with respect to the doppler width
	double compute_dRho_Q__dDldop();
	/// Compute derivative of Rho_Q with respect to the damping
	double compute_dRho_Q__da();
	/// Compute derivative of Rho_Q with respect to the wavelength shift
	double compute_dRho_Q__dv();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	
	/// Derivatives of Rho_U (element of the absortion matrix)    
	double dRho_U__dInclination;
	double dRho_U__dB;
	double dRho_U__dAzimuth;
	double dRho_U__dEta_0;
	double dRho_U__dDldop;
	double dRho_U__da;
	vector<double> dRho_U__dv;
	/// Compute derivative of Rho_U with respect to the field
	double compute_dRho_U__dB();
	/// Compute derivative of Rho_U with respect to the inclination
	double compute_dRho_U__dInclination();
	/// Compute derivative of Rho_U with respect to the azimuth
	double compute_dRho_U__dAzimuth();
	/// Compute derivative of Rho_U with respect to the line strength
	double compute_dRho_U__dEta_0();
	/// Compute derivative of Rho_U with respect to the doppler width
	double compute_dRho_U__dDldop();
	/// Compute derivative of Rho_U with respect to the damping
	double compute_dRho_U__da();
	/// Compute derivative of Rho_U with respect to the wavelength shift
	double compute_dRho_U__dv();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	/// Derivatives of Rho_V (element of the absortion matrix) 
	double dRho_V__dB;
	double dRho_V__dInclination;	
	double dRho_V__dEta_0;
	double dRho_V__dDldop;
	double dRho_V__da;
	vector<double> dRho_V__dv;
	/// Compute derivative of Rho_V with respect to the field
	double compute_dRho_V__dB();
	/// Compute derivative of Rho_V with respect to the inclination
	double compute_dRho_V__dInclination();
	/// Compute derivative of Rho_V with respect to the line strength
	double compute_dRho_V__dEta_0();
	/// Compute derivative of Rho_V with respect to the doppler width
	double compute_dRho_V__dDldop();
	/// Compute derivative of Rho_V with respect to the damping
	double compute_dRho_V__da();
	/// Compute derivative of Rho_V with respect to the wavelength shift
	double compute_dRho_V__dv();
	///////////////////////////////////////////////////////////////////////////////////////////
	
	
	/// Derivatives of the absortion and dispersion profiles
	/// with respect to the field, the wavelength shift,
	/// the damping and the doppler width.
	/// Notice that the component PI of the absortion profile
	/// does not depend on the magnetic field.
	double dPhi_p__dv;
	double dPhi_r__dv;
	double dPhi_b__dv;
	double dPhi_p__dB;
	double dPhi_r__dB;
	double dPhi_b__dB;
	double dPhi_p__da;
	double dPhi_r__da;
	double dPhi_b__da;
	double dPhi_p__dDldop;
	double dPhi_r__dDldop;
	double dPhi_b__dDldop;
	double dPsi_p__dB;
	double dPsi_b__dB;
	double dPsi_r__dB;
	double dPsi_p__dv;
	double dPsi_r__dv;
	double dPsi_b__dv;
	double dPsi_p__da;
	double dPsi_b__da;
	double dPsi_r__da;
	double dPsi_p__dDldop;
	double dPsi_b__dDldop;
	double dPsi_r__dDldop;
	
	/// These are just synthetic variables to hold values
	/// of already computed quantities to avoid recalculation
	
	/// Trigonometric values				
	double Sin_inc_square;			/// square of the sine of the inclination 
	double Cos_inc_square;			/// square of the cosine of the inclination
	double Sin_twice_azi;			/// sine of 2*azimuth
	double Cos_twice_azi; 			/// cosine of 2*azimuth
	double Sin_inc;					/// sine of the inclination
	double Cos_inc;					/// cosine of the inclination
	
	/// Common factors in the many equations
	double common_factor0;			/// common_factor0 = Eta_0 * .5
	double common_factor1;			/// common_factor1 = .5 * (Phi_b + Phi_r)
        double common_factor2;			/// common_factor2 = Phi_p - common_factor1
        double common_factor3;			/// common_factor3 = Sin_inc_square * Cos_twice_azi
	double common_factor4;			/// common_factor4 = Sin_inc_square * Sin_twice_azi
	double common_factor5;			/// common_factor5 = .5 * (Psi_b + Psi_r) 
	double common_factor6;			/// common_factor6 = Psi_p - common_factor5
	double common_factor7;			/// common_factor7 = common_factor0 * common_factor3
	double common_factor8;			/// common_factor8 = common_factor0 * common_factor4
	double common_factor9;			/// common_factor9 = 1. + Cos_inc_square
	double common_factor10;			/// common_factor10 = (Phi_r - Phi_b)
	double common_factor11;			/// common_factor11 = (Psi_r - Psi_b)
	double common_factor12;			/// common_factor12 = Eta_I_square + Rho_Q_square + Rho_U_square + Rho_V_square
	
	/// Common factors part of the DELTA part of the equations
	double det1;
	double det3;
	double B1_divided_by_Delta;
	double B1_divided_by_Delta_square;
	
	/// Square Power of elements of the absortion matrix and the denominator Delta
	double Eta_I_square; /// Eta_I * Eta_I
	double Eta_Q_square; /// Eta_Q * Eta_Q
	double Eta_U_square; /// Eta_U * Eta_U
	double Eta_V_square; /// Eta_V * Eta_V
	double Rho_U_square; /// Rho_U * Rho_U
	double Rho_Q_square; /// Rho_Q * Rho_Q
	double Rho_V_square; /// Rho_V * Rho_V
	double Delta_square; /// Delta * Delta
	
    public:
		
        /// Constructor
        MERLIN_MilneEddington() {}
	/// Destructor
	~MERLIN_MilneEddington() {}
	/// Obtain the profile for provided Model object.
	void get_profile (MERLIN_Model &model, MERLIN_Profile &profile, vector< vector<double> > * absortion_matrix_elements=0);
	/// Obtain the profile and its derivatives for provided Model object.
	void get_profile_and_derivatives(MERLIN_Model &model, MERLIN_Profile &profile);
	///
	void apply_filter(){}
  
    }; // end class MERLIN_MilneEddington

} // end namespace CSAC

#endif // MERLIN_MilneEddington_h_

