/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Model.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#include <string>
using std::string;
#include <fstream>
using std::ifstream;
using std::getline;
#include <iostream>
using std::cout;
using std::endl;
#include <sstream>
using std::stringstream;

#include "MERLIN_Model.h"
using CSAC::MERLIN_Model;

void MERLIN_Model::load_regions_and_spectral_lines( const string &s_grid_file_name )
{
	
    const char* grid_file_name = s_grid_file_name.c_str();
	
    ifstream grid_file( grid_file_name ) ;
    if( !grid_file )
    {
        string err = string("The grid file ") + grid_file_name + " does not exist or cannot be opened" ;
        throw err ;
    }
    if( grid_file.eof() )
    {
        string err = string("The grid file ") + grid_file_name + " is empty" ;
        throw err ;
    }
	
    string buffer ;
	
    // grab the first line, which tells us how man regions there are and how
    // many spectral lines there are
    if( getline( grid_file, buffer ) )
    {
        stringstream ss( buffer ) ;
        ss >> n_regions ;
        ss >> n_lines ;
        if( n_regions == 0 )
        {
            string err = string( "The number of regions cannot be 0 ")
            + " in grid file " + grid_file_name ;
            throw string( err ) ;
        }
        if( n_lines == 0 )
        {
            string err = string( "The number of spectral lines cannot be 0 ")
            + " in grid file " + grid_file_name ;
            throw string( err ) ;
        }
    }
	
    // The next lines are the region information given the number of regions
    // specified on the first line.
    try
    {
        for( int region_index = 0; region_index < n_regions; region_index++ )
        {
            getline( grid_file, buffer ) ;
            if( grid_file.eof() )
            {
                stringstream err ;
                err << "Error reading regions in grid file "
                    << grid_file_name << ": " ;
                err << "read " << region_index << " regions, " ;
                err << "trying to read " << n_regions << " regions" ;
                throw err.str() ;
            }
            parse_region( buffer ) ;
        }
    }
    catch( string &e )
    {
        string err = string( "Error reading grid file " )
        + grid_file_name + ":\n" + e ;
        throw err ;
    }
	
	
    // The last set of lines are the spectral line information given the
    // number of spectral lines specified on the first line of the file
    try
    {
        for( int line_index = 0; line_index < n_lines; line_index++ )
        {
            getline( grid_file, buffer ) ;
            if( grid_file.eof() )
            {
                stringstream err ;
                err << "Error reading spectral lines in grid file "
                    << grid_file_name << ": " ;
                err << "read " << line_index << " lines, " ;
                err << "trying to read " << n_lines << " lines" ;
                throw err.str() ;
            }
            parse_spectral( buffer ) ;
        }
    }
    catch( string &e )
    {
        string err = string( "Error reading grid file " )
        + grid_file_name + ":\n" + e ;
        throw err ;
    }
	
    // Make sure we are at the end of the file, that there is no more
    // information in the file. If there is, throw an exception
    getline( grid_file, buffer ) ;
    if( !grid_file.eof() )
    {
        string err = string( "too many lines in grid file " )
        + grid_file_name ;
        throw string( err ) ;
    }
	
    grid_file.close() ;
	
    for( int region_index = 0; region_index < n_regions; region_index++ )
        n_total_wave += regions[region_index].n_wavelengths;
}

void
MERLIN_Model::parse_region( string &r_line )
{
    MERLIN_Region r ;
	
    stringstream ss( r_line ) ;
	
    ss >> r.first_wavelength ;
    ss >> r.wave_step ;
    ss >> r.n_wavelengths ;
	
    if ( r.first_wavelength == 0.0 ||
         r.wave_step == 0.0         ||
         r.n_wavelengths == 0 
         )
    {
        string err( "Incorrect syntax in region line " ) ;
        err += r_line ;
        throw err ;
    }
	
    regions.push_back( r ) ;
}

void
MERLIN_Model::parse_spectral( string &s_line )
{
    stringstream ss( s_line ) ;
    MERLIN_AtomicLine l ;
	
    ss >> l.central_wav ;
	
    try
    {
        string e_lower_str ;
        ss >> e_lower_str ;
        MERLIN_LScoupling	e_lower( e_lower_str.c_str() ) ;
        l.e_lower = e_lower ;
    }
    catch( string &c_err )
    {
        string err( "Incorrect syntax in spectral line " ) ;
        err += s_line + "\n" + c_err ;
        throw err ;
    }
	
    try
    {
        string e_upper_str ;
        ss >> e_upper_str ;
        MERLIN_LScoupling	e_upper( e_upper_str.c_str() ) ;
        l.e_upper = e_upper ;
    }
    catch( string &c_err )
    {
        string err( "Incorrect syntax in spectral line " ) ;
        err += s_line + "\n" + c_err ;
        throw err ;
    }
	
    ss >> l.relative_strength ;
	
    if ( 
        l.central_wav == 0.0 ||
        l.relative_strength == 0.0 
         )
    {
        string err( "Incorrect syntax in spectral line values" ) ;
        err += s_line ;
        throw err ;
    }
	
    l.load_components();
	
    lines.push_back( l ) ;
}

void
MERLIN_Model::dump( ostream &strm ) const
{
    strm.precision(15);
    strm << "MERLIN_Model: " << (void *)this << ":" << endl ;
    strm<<"    n_total_wave = " << n_total_wave << endl;
    // dump information about the regions
    strm << "    n_regions = " << n_regions << endl ;
    strm << "    regions:" << endl ;
    vector<MERLIN_Region>::const_iterator r_iter ;
    for( r_iter = regions.begin(); r_iter != regions.end(); r_iter++ )
        strm<<(*r_iter);
    // dump information about the lines
    strm << "    n_lines = " << n_lines << endl ;
    strm << "    spectral lines:" ;
    vector<MERLIN_AtomicLine>::const_iterator l_iter ;
    for( l_iter = lines.begin(); l_iter != lines.end(); l_iter++ )
        strm<<(*l_iter);
    // dump information about the magnetic field
    strm<<"Magnetic Field (Gauss): "<<magnetic_field.value<<", free? ";
    if( magnetic_field.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the inclination
    strm<<"Inclination (radians): "<<inclination.value<<", (degrees): "<<inclination.value * 180 / CSAC::Pi<<", free? ";
    if( inclination.free )
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the azimuth	
    strm<<"Azimuth (radians): "<<azimuth.value<<", (degrees): "<<azimuth.value * 180 / CSAC::Pi<<", free? ";
    if( azimuth.free )
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the Line Strength
    strm<<"Line Strength, aka Eta 0 (dimensionless): "<<line_strength.value<<", free? ";
    if( line_strength.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the Doppler Width 
    strm<<"Doppler Width (milli Angstroms):  "<<doppler_width.value<<", free? ";
    if( doppler_width.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the Damping relative to Doppler width
    strm<<"Damping relative to Doppler (dimensionless): "<<damping.value<<", free? ";
    if( damping.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the Plack Function
    strm<<"Plack Function, aka B0 (same units as intensity): "<<planck_function.value<<", free? ";
    if( planck_function.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the Plack Function Gradient	
    strm<<"Plack Function Gradient, aka B1 (same units as intensity): "<<planck_function_gradient.value<<", free? ";
    if( planck_function_gradient.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about the macroturbulence
    strm<<"Macroturbulence (km/s): "<<vmac.value<<", free? ";
    if( vmac.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
	
    // dump information about the stray light vector
    if(stray_light_profile.set())
    {
        strm<<"This run was provided with a stray light profile: "<<endl;
        strm<<"--------------------------------------------------->"<<endl;
        strm<<stray_light_profile;
        strm<<"Lenght of the stray light profile = "<<stray_light_profile.size()<<endl;
        strm<<"--------------------------------------------------->"<<endl;
    }
	
    // dump information about the stray light fraction
    strm<<"Stray Light (0->1): "<<stray_light.value<<", free? ";
    if( stray_light.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
    // dump information about thestray light fraction shift
    strm<<"Stray Light shift (km/s): "<<stray_light_shift.value<<", free? ";
    if( stray_light_shift.free)
        strm<<"true"<<endl;
    else
        strm<<"false"<<endl;
	
    strm<<"Total number of free parameters: "<<n_total_free_parameters<<endl;
}

