/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Model.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_Model_h_
#define MERLIN_Model_h_ 1

#include <vector>
using std::vector;
#include <string>
using std::string;

#include "MERLIN.h"
#include "MERLIN_Object.h"
using CSAC::MERLIN_Object;
#include "MERLIN_Region.h"
using CSAC::MERLIN_Region;
#include "MERLIN_AtomicLine.h"
using CSAC::MERLIN_AtomicLine;
#include "MERLIN_Parameter.h"
using CSAC::MERLIN_Parameter;
#include "MERLIN_StrayLight.h"
using CSAC::MERLIN_StrayLight;

namespace CSAC
{
  
  
    /// Group of parameters that defined a Milne-Eddington model.
    /**
       This class contains all the information for a Milne Eddington Atmosphere, 
       the stray light, the shift of the stray light, the stray light fraction, 
       the regions of operation, the information about all the spectral lines involve, etc.
    */
    class MERLIN_Model : public MERLIN_Object
    {
    private:
	///
	void parse_region( string &r ) ;
	///
	void parse_spectral( string &s ) ;
	/// 
	typedef struct istokes_
	{
            bool			invert_I ;
            bool			invert_U ;
            bool			invert_Q ;
            bool			invert_V ;
	} istokes ;
    public:
        /// Set to true which vectors are to be computed/inverted 
        istokes vectors_to_invert ;
	/// Total number of waves is the sum of the n_wavelengths for all regions. 
	unsigned int n_total_wave ;
	/// Total number of free parameters
	unsigned int n_total_free_parameters;
	/// Number of active regions
	int n_regions ; 
	/// Number of atomic lines
	int n_lines ;
	/// Regions of wave lenghts  
	vector<MERLIN_Region>	regions ;
	/// Atomic Lines used in the model
	vector<MERLIN_AtomicLine>	lines ;
	///  Magnetic field strength (Gauss)
	MERLIN_Parameter magnetic_field ;        
	///  Inclination (degress)     
	MERLIN_Parameter inclination ;                
	///  Azimuth (degress)
	MERLIN_Parameter azimuth ;                    
	///  Line strength (dimensionless)
	MERLIN_Parameter line_strength ;              
	///  Doppler width (milli Angstroms)
	MERLIN_Parameter doppler_width ;              
	///  Damping (dimensionless)
	MERLIN_Parameter damping;          
	///  Planck function (same units as intensity)
	MERLIN_Parameter planck_function ;            
	///  Planck function gradient (same units as intensity)
	MERLIN_Parameter planck_function_gradient ;  
	/// Macroturbulence (km/s)
	MERLIN_Parameter vmac;
	/// Stray light (0 -> 1)
	MERLIN_Parameter stray_light ;   
	/// Shift on the stray light 
	MERLIN_Parameter stray_light_shift;            
	/// Stray light profile
	MERLIN_StrayLight stray_light_profile ;
	/// Minimum space between regions, between first region to end and between last region to end
	double delta_min;
    public:
        /// Default constructor.
        MERLIN_Model(): 
            n_regions( 0 ),
            n_lines( 0 ),
            n_total_wave(0),
            n_total_free_parameters(0)
        {
            vectors_to_invert.invert_I = true;
            vectors_to_invert.invert_U = true;
            vectors_to_invert.invert_Q = true;
            vectors_to_invert.invert_V = true;
        }
  
	/// Copy constructor
	MERLIN_Model(const MERLIN_Model &rhs)
        {
            //cout<<"Using MERLIN_Model copy constructor"<<endl;
            n_total_wave=rhs.n_total_wave;
            n_total_free_parameters=rhs.n_total_free_parameters;
            vectors_to_invert=rhs.vectors_to_invert;
            n_regions=rhs.n_regions;
            n_lines=rhs.n_lines;
            regions.resize(rhs.regions.size());
            for (int i=0; i < rhs.regions.size(); i++)
                regions[i]=rhs.regions[i];
            lines.resize(rhs.lines.size());
            for (int i=0; i < rhs.lines.size(); i++)
                lines[i]=rhs.lines[i];
            magnetic_field=rhs.magnetic_field;
            inclination=rhs.inclination;
            azimuth=rhs.azimuth;
            line_strength=rhs.line_strength;
            doppler_width=rhs.doppler_width;
            damping=rhs.damping;
            planck_function=rhs.planck_function;
            planck_function_gradient=rhs.planck_function_gradient;
            vmac=rhs.vmac;
            stray_light=rhs.stray_light;
            stray_light_shift=rhs.stray_light_shift;
            stray_light_profile= rhs.stray_light_profile;
        }
	/// Destructor
	virtual ~MERLIN_Model() {}
	/// \brief Obtain data about the spectral lines and the regions from a file.
        ///
        ///  Representation of the file structure for parsing regions and spectral lines out of a file.
        ///  
        ///  The grid file looks like the following
        ///  <pre>
        ///  1  2
        ///  6301   12.8    200
        ///  6301.5 5P2 5D2 1.
        ///  6302.5 5P1 5D0 0.342466
        ///  </pre>
        ///  
        ///  The first line has two values, the number of regions followed by the
        ///  number of lines (i.e., spectral lines)
        ///  
        ///  Since in this example there is only one region, the next line has in
        ///  this order:
        ///  First_wavelength Wave_step n_wavelengths
        ///  
        ///  The remaining lines in the file are the spectral lines of the format:
        ///  Central_wave LS_coupling lower LS_coupling upper and the Relative_strength
        ///  
        ///  @see MERLIN_LScoupling
	void load_regions_and_spectral_lines (const string &) ;
	/// Printer method
	virtual void dump( ostream &strm ) const ;
	/// Overloaded assigment operator.
	MERLIN_Model& operator=(const MERLIN_Model &rhs)
        {
    
            if(this == &rhs)
                return *this;
            //cout<<"Using MERLIN_Model overloaded ="<<endl;
            n_total_wave=rhs.n_total_wave;
            n_total_free_parameters=rhs.n_total_free_parameters;
            vectors_to_invert=rhs.vectors_to_invert;
            n_regions=rhs.n_regions;
            n_lines=rhs.n_lines;
            regions.resize(rhs.regions.size());
            for (int i=0; i < rhs.regions.size(); i++)
                regions[i]=rhs.regions[i];
            lines.resize(rhs.lines.size());
            for (int i=0; i < rhs.lines.size(); i++)
                lines[i]=rhs.lines[i];
            magnetic_field=rhs.magnetic_field;
            inclination=rhs.inclination;
            azimuth=rhs.azimuth;
            line_strength=rhs.line_strength;
            doppler_width=rhs.doppler_width;
            damping=rhs.damping;
            planck_function=rhs.planck_function;
            planck_function_gradient=rhs.planck_function_gradient;
            vmac=rhs.vmac;
            stray_light=rhs.stray_light;
            stray_light_shift=rhs.stray_light_shift;
            stray_light_profile= rhs.stray_light_profile;
    
            return *this;
        }
        /// set variable n_total_free_parameters to the total number of free parameters.
	/// this method must be called ***ONLY*** after the free parameters, the spectral lines
	/// and the regions has been loaded into the object.
	void set_n_total_free_parameters()
        {
            n_total_free_parameters =0;
            for (int g=0; g<lines.size(); g++)
                if (lines[g].wavelength_shift.free)
                    n_total_free_parameters++; 
    
            if (magnetic_field.free)  n_total_free_parameters++;        
    
            if (inclination.free)  n_total_free_parameters++ ;                
    
            if (azimuth.free)  n_total_free_parameters++ ;                    
    
            if (line_strength.free)  n_total_free_parameters++ ;              
    
            if (doppler_width.free)  n_total_free_parameters++ ;              
    
            if (damping.free)  n_total_free_parameters++ ;          
    
            if (planck_function.free)  n_total_free_parameters++ ;            
    
            if (planck_function_gradient.free)  n_total_free_parameters++ ;  
    
            if (vmac.free)  n_total_free_parameters++;
    
            if (stray_light.free)  n_total_free_parameters++ ;   
    
            if (stray_light_shift.free)  n_total_free_parameters++;   
        }    
  
    };  

} // end namespace CSAC

#endif //  MERLIN_Model_h_
