/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ParametersLimits.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include "MERLIN_ParametersLimits.h"

using CSAC::MERLIN_ParametersLimits;

MERLIN_ParametersLimits::MERLIN_ParametersLimits():
    _max_planck_function_set(false),
    _max_planck_function_gradient_set(false) 
{            
    //  Minimum Line strength (dimensionless)
    _min_line_strength = 1E-2;              
    //  Minimum Doppler width (milli Angstroms)
    _min_doppler_width = 20.0;              
    //  Minimum Damping (dimensionless)
    _min_damping = 0.0;          
    ///  Minimum Planck function (same units as intensity)
    _min_planck_function = 0.0;            
    ///  Minimum Planck function gradient (same units as intensity)
    _min_planck_function_gradient = 0.0;
    // Minimum Macroturbulence (km/s)
    _min_vmac =  0.7083;
    // Minimum Stray light (0 -> 1)
    _min_stray_light =0.0;   
    // Minimum Shift on the stray light 
    _min_stray_light_shift =-5.0;            
    //  Maximum Magnetic field strength (Gauss)
    _max_magnetic_field = 5000.0;        
    //  Maximum Line strength (dimensionless)
    _max_line_strength = 100.0;              
    //  Maximum Doppler width (milli Angstroms)
    _max_doppler_width = 90.0;              
    //  Maximum Damping (dimensionless)
    _max_damping = 1.50;          
    // Maximum Macroturbulence (km/s)
    _max_vmac = 3.0;
    // Maximum Stray light (0 -> 1)
    _max_stray_light = 1.0;   
    // Maximum Shift on the stray light 
    _max_stray_light_shift = 5.0;  
}      


//
void 
MERLIN_ParametersLimits::set_max_magnetic_field(const double & val) throw (string)
{
    if (val < 0)
        throw string("STOP! Invalid value for setting the maximun of strength for the magnetic field. Value less than zero.");
    _max_magnetic_field = val;
}

//
void 
MERLIN_ParametersLimits::set_min_line_strength(const double & val) throw (string)
{
    _min_line_strength = val;
}

//
void 
MERLIN_ParametersLimits::set_max_line_strength(const double & val) throw (string)
{
    if (val < _min_line_strength)
        throw string("STOP! Invalid value for setting maximun line strength. Value less than minimum line strength.");
    _max_line_strength = val;
}

//
void 
MERLIN_ParametersLimits::set_min__doppler_width(const double & val) throw (string)
{
    _min_doppler_width = val;
}

//
void 
MERLIN_ParametersLimits::set_max_doppler_width(const double & val) throw (string)
{
    if (val < _min_doppler_width)
        throw string("STOP! Invalid value for setting the doppler width. Value less than minimum doppler width.");
    _max_doppler_width = val;
	
}

//
void 
MERLIN_ParametersLimits::set_min_damping(const double & val) throw (string)
{
    if (val < 0)
        throw string("STOP! Invalid Value for minimum damping. Value is less than zero.");
    _min_damping = val;
}

//
void 
MERLIN_ParametersLimits::set_max_damping(const double & val) throw (string)
{
    if (val < _min_damping)
        throw string("STOP! Invalid maximum value for damping. Value is less than minimum damping.");
    _max_damping = val;
}

//
void 
MERLIN_ParametersLimits::set_min_vmac(const double & val) throw (string)
{
    if (val < 0)
        throw string("STOP! Invalid minimum value for the macroturbulence. Value is less than zero.");
    _min_vmac = val;
}

//
void 
MERLIN_ParametersLimits::set_max_vmac(const double & val) throw (string)
{
    if (val < _min_vmac)
        throw string("STOP! Invalid maximum value for the macroturbulence. Value is less than the minimum value for the macroturbulence.");
    if (val > 3.0)
        throw string("STOP! Invalid maximum value for the macroturbulence. Value is greater than the 3.0 Km/s.");	
    _max_vmac = val;
}

///
void 
MERLIN_ParametersLimits::set_min_planck_function(const double & val) throw (string)
{
    if (val < 0)
        throw string("STOP! Invalid minimum value for the planck function B0. Value is less than zero.");
    _min_planck_function = val;
}

///
void 
MERLIN_ParametersLimits::set_min_planck_function_gradient(const double & val) throw (string)
{
    if (val < 0)
        throw string("STOP! Invalid minimum value for the planck function gradient (B1). Value is less than zero.");
    _min_planck_function_gradient = val;
}

///
void 
MERLIN_ParametersLimits::set_max_planck_function(const double & val) throw (string)
{
    if (val < _min_planck_function)
        throw string("STOP! Invalid maximun value for the planck function B0. Value is less than minimum value for the planck function B0.");
    _max_planck_function = val;
		
    _max_planck_function_set = true;
}

///
void 
MERLIN_ParametersLimits::set_max_planck_function_gradient(const double & val) throw (string)
{
    if (val < _min_planck_function_gradient)
        throw string("STOP! Invalid maximun value for the planck function gradient (B1). Value is less than minimum value for the planck function gradient (B1).");
    _max_planck_function_gradient = val;
		
    _max_planck_function_gradient_set = true;
}

