/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ZeemanSplitting.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */



#include <string>
using std::string;

#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))

extern "C" {
    void zeeman_ (
                  /* Inputs values*/
                  int *MC,       /* Max dimension of the vectors DLP, DLL, DLR, SP, SL, SR */
                  int *MULT,     /* MULT: MULT[0] is 2*SL+1 and MULT[1] is 2*SU+1 */
                  char *DESIGN,  /* DESIGN[1] is 'S' if LL=0, 'P' if LL=1, ... etc 
                                    DESIGN[2] is 'S' if LU=0, 'P' if LU=1, ... etc  */
                  float *TAM,    /* TAM: TAM[1] es JL, TAM[2] es JU */
                  int *JI,       /* JI[i] is the integer part of TAM[i] */
                  int *JF,       /* JF[i] is the decimal part of TAM[i] */
                  float *DL0,    /* DL0: formula que usa la longitud de onda en amstromg y la devuelve en mA */  
                  /* Return values */
                  int *NP,       /* Number of PI transitions */
                  int *NL,       /* Number of sigma left transitions */
                  int *NR,       /* Number of sigma right transitions */
                  float *DLP,    
                  float *DLL,
                  float *DLR,
                  float *SP,
                  float *SL,
                  float *SR
                  );
	
    void zeeman_comp_(
                       /* Inputs values*/
                       double *BFIELD, /* INTENSITY OF THE MAGNETIC FIELD IN GAUSS */
                       double *LANDA0, /* WAVE FOR WHICH THE ZEEMAN SPLITTING IS HAPPENING IN AMSTROMG */
                       int *JL,        /* LOWER TOTAL ANGULAR MOMENTUM  */
                       int *JU,        /* UPPER TOTAL ANGULAR MOMENTUN  */
                       int *SL,        /* LOWER SPIN ANGULAR MOMENTUM  */
                       int *SU,        /* UPPER SPIN ANGULAR MOMENTUM  */
                       int *LL,        /* LOWER ORBITAL ANGULAR MOMENTUM  */
                       int *LU,        /* UPPER ORBITA; ANGULAR MOMENTUM  */
                       int *NTOT,      /* TOTAL NUMBER OF TRANSITIONS (NTOT = 2*MIN(JU,JL)+1+JU+JL+JU+JL) */
                       /* Return values */
                       double *ENG,    /* ENG -> VECTOR SIZE NTOT. FULL ZEEMAN PATTERN ORDERED IN WAVELENGTH */
                       double *STR,    /* STR -> VECTOR SIZE NTOT. NORMALIZED STRENGTH OF THE ZEEMAN PATTERN */  
                       double *LOC,    /* LOC -> VECTOR SIZE NTOT. LOCALIZATOR FOR SIGMA OR PI TRANSITIONS   */
                       double *DEDB    /* DEDB -> VECTOR SIZE NTOT. DERIVATIVE OF ENG WITH RESPECT TO THE MAGNETIC FIELD */
                       );
}

#include "MERLIN_ZeemanSplitting.h"
using CSAC::MERLIN_ZeemanSplitting;

void 
MERLIN_ZeemanSplitting::compute
(
 double &wavelength, 
 MERLIN_LScoupling &e_lower, 
 MERLIN_LScoupling &e_upper, 
 vector<MERLIN_ZeemanComponent> &components
 )
{
    int MC=20;
    int MULT[2];
    MULT[0]=e_lower.get_2Splus1();
    MULT[1]=e_upper.get_2Splus1();
	
    char DESIGN[2];
    DESIGN[0]=e_lower.get_Lchar();
    DESIGN[1]=e_upper.get_Lchar();
	
    float TAM[2];
    TAM[0]=e_lower.get_J();
    TAM[1]=e_upper.get_J();
	
    int JI[2];
    int JF[2];
    JI[0]=e_lower.get_Jint();
    JI[1]=e_upper.get_Jint();
    JF[0]=e_lower.get_Jdec();
    JF[1]=e_upper.get_Jdec();
	
    /* float  Dlo=4.6686e-5*(Wavelength*1e-8)*(Wavelength*1e-8) Lambda_B in cm */
    float DL0=4.6686e-13*wavelength*wavelength*1.e3; // in mA
	
    int NP, NL, NR;
	
	float *DLP, *DLL, *DLR, *SP, *SL, *SR;
    DLP = new float [MC];
    DLL = new float [MC];
    DLR = new float [MC];
    SP = new float [MC];
    SL = new float [MC];
    SR = new float [MC];
	
    for (int j=0; j<MC; j++)
    {
        DLP[j]=0.;
        DLL[j]=0.;
        DLR[j]=0.;
        SP[j]=0.;
        SL[j]=0.;
        SR[j]=0.;
    }
	
    zeeman_(&MC, MULT, DESIGN, TAM, JI, JF, &DL0, &NP, &NL, &NR, DLP, DLL, DLR, SP, SL, SR);
	
    MERLIN_ZeemanComponent comp;
    for (int h=0; h < NP; h++)
    {
        comp.type = MERLIN_ZeemanComponent::PI;
        comp.strength = SP[h];
        comp.derivative = DLP[h];
        components.push_back(comp);
    }
    for (int h=0; h<NR; h++)
    {
        comp.type = MERLIN_ZeemanComponent::SIGMA_RIGHT;
        comp.strength = SR[h];
        comp.derivative = DLR[h];
        components.push_back(comp);
    }
    for (int h=0; h<NL; h++)
    {
        comp.type = MERLIN_ZeemanComponent::SIGMA_LEFT;
        comp.strength = SL[h];
        comp.derivative = DLL[h];
        components.push_back(comp);
    }
	
	delete [] DLP;
	delete [] DLL;
	delete [] DLR;
	delete [] SP;
	delete [] SL;
	delete [] SR;
};

void 
MERLIN_ZeemanSplitting::compute
(
 double &wavelength, 
 MERLIN_LScoupling &e_lower, 
 MERLIN_LScoupling &e_upper, 
 vector<MERLIN_ZeemanComponent> &components, 
 vector<double> &full_spectrum
 )
{
    double BFIELD=1.0;
	
    int SL_=e_lower.get_S();
    int SU_=e_upper.get_S();
    int LL_=e_lower.get_L();
    int LU_=e_upper.get_L();
    int JL_=e_lower.get_Jint();
    int JU_=e_upper.get_Jint(); 
	
	
    int NTOT = 2*MIN(JU_,JL_)+1+JU_+JL_+JU_+JL_;
	
	double *ENG, *STR, *LOC, *DEDB;
    ENG = new double [NTOT];
    STR = new double [NTOT];
    LOC = new double [NTOT];
    DEDB = new double [NTOT];
	
    zeeman_comp_(&BFIELD, &wavelength, &JL_, &JU_, &SL_, &SU_, &LL_, &LU_, &NTOT, ENG, STR, LOC, DEDB);
	
    MERLIN_ZeemanComponent comp;
    for (int l=0; l<NTOT; l++)
    {
        comp.strength=STR[l];
        comp.derivative=DEDB[l];
        if (LOC[l] == 1.)
            comp.type = MERLIN_ZeemanComponent::SIGMA_RIGHT;
        else if (LOC[l] == 0.)
            comp.type = MERLIN_ZeemanComponent::PI;
        else if (LOC[l] == -1.)
            comp.type = MERLIN_ZeemanComponent::SIGMA_LEFT;
        else
            throw string("Invalid type of Zeeman component");
        components.push_back(comp);
        full_spectrum.push_back(ENG[l]);
    }
	
	delete [] ENG;
	delete [] STR;
	delete [] LOC;
	delete [] DEDB;
}
